# -*- coding: utf-8 -*-
# Andrea Castiella Aguirrezabala
# Loudness ISO 532-1:2014
# Método para sonidos estacionarios y variables con el tiempo

import numpy as np

constantes = {
    'SR_LEVEL': 2000,
    'N_BARK_BANDS': 240,
    'N_LCB_BANDS': 11,
    'N_CORE_LOUDN': 21,
    'N_RAP_RANGES': 8,
    'N_RNS_RANGES': 18,
    'N_CB_RANGES': 8,
    'NL_ITER': 24,
    'LP_ITER': 24,
    'SOUNDFIELDDIFFUSE': 1,
    'SOUNDFILEDFREE': 0,
    'TSHORT': 0.005,
    'TLONG': 0.015,
    'TVAR': 0.075,
    'DEC_FACTOR': 4 # Pasar de 0.5 ms a 2 ms
}

variablesBark = {
    'N_BARK_BANDS': 24
}


# Corrección para bajas frecuencias
def f_corr_third_octave_intensities(ThirdOctaveLevel):
    ThirdOctaveIntens = np.zeros(11)
    # Por número de LCB BANDS de 25 a 250 Hz
    for idxIntens in range(constantes['N_LCB_BANDS']):
        # Ponderación de los niveles de un tercio de octava para bandas entre 25 y 250 Hz
        DLL = np.array([[-32, -24, -16, -10, -5, 0, -7, -3, 0, -2, 0],
                        [-29, -22, -15, -10, -4, 0, -7, -2, 0, -2, 0],
                        [-27, -19, -14, -9, -4, 0, -6, -2, 0, -2, 0],
                        [-25, -17, -12, -9, -3, 0, -5, -2, 0, -2, 0],
                        [-23, -16, -11, -7, -3, 0, -4, -1, 0, -1, 0],
                        [-20, -14, -10, -6, -3, 0, -4, -1, 0, -1, 0],
                        [-18, -12, -9, -6, -2, 0, -3, -1, 0, -1, 0],
                        [-15, -10, -8, -4, -2, 0, -3, -1, 0, -1, 0]])
        # Rangos de nivel para ponderación de los niveles
        RAP = np.array([45, 55, 65, 71, 80, 90, 100, 120])

        # Corrección primeras 11 bandas 
        idxLevelRange = 0
        while (ThirdOctaveLevel[idxIntens] > RAP[idxLevelRange] - DLL[idxLevelRange][idxIntens]) and (
                idxLevelRange < constantes['N_RAP_RANGES'] - 1):
            idxLevelRange += 1
        CorrLevel = ThirdOctaveLevel[idxIntens] + DLL[idxLevelRange][idxIntens]
        ThirdOctaveIntens[idxIntens] = np.power(10, CorrLevel / 10)
    return ThirdOctaveIntens


# Niveles de las primeras bandas criticas
def f_calc_lcbs(ThirdOctaveIntens):
    # Aproximación a bandas críticas por debajo de 300 Hz.
    LCB = np.zeros(3)
    pcbi = np.zeros(3)
    for idxIntens in range(6):
        pcbi[0] += ThirdOctaveIntens[idxIntens]
    for idxIntens in range(6, 9):
        pcbi[1] += ThirdOctaveIntens[idxIntens]
    for idxIntens in range(9, 11):
        pcbi[2] += ThirdOctaveIntens[idxIntens]
    for i in range(3):
        if pcbi[i] > 0:
            LCB[i] = 10 * np.log10(pcbi[i])
    return LCB


# Cálculo del core loudness
def f_calc_core_loudness(ThirdOctaveSPL, LCB, SoundFieldDiffuse=constantes['SOUNDFIELDDIFFUSE']):
    coreLoudness = np.zeros(constantes['N_CORE_LOUDN'])
    criticalBandLevels = np.zeros(constantes['N_CORE_LOUDN'])
    # Características de transmisión del oído humano
    A0 = np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.5, -1.6, -3.2, -5.4, -5.6, -4.0, -1.5, 2.0, 5.0, 12.0])
    # Diferencia de nivel para campo difuso
    DDF = np.array([0, 0, 0.5, 0.9, 1.2, 1.6, 2.3, 2.8, 3.0, 2.0, 0, -1.4, -2.0, -1.9, -1.0, 0.5, 3.0, 4.0, 4.3, 4.0])
    # Umbrales de silencio
    LTQ = np.array([30, 18, 12, 8, 7, 6, 5, 4, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3])
    # Corrección por aproximación
    DCB = np.array([-0.25, -0.6, -0.8, -0.8, -0.5, 0, 0.5, 1.1, 1.5, 1.7, 1.8, 1.8, 1.7, 1.6, 1.4, 1.2, 0.8, 0.5, 0,
                    -0.5])
    # Factor umbral
    S = 0.25

    for idxCL in range(constantes['N_CORE_LOUDN'] - 1):
        if idxCL < 3:
            Le = LCB[idxCL]
        else:
            Le = ThirdOctaveSPL[idxCL + 8]

        Le = Le - A0[idxCL]
        coreLoudness[idxCL] = 0

        # Corrección por campo difuso
        if SoundFieldDiffuse == 1:
            Le = Le + DDF[idxCL]

        # Correción si se supera el umbral de silencio
        if Le > LTQ[idxCL]:
            Le = Le - DCB[idxCL]
            C1 = 0.0635 * 10 ** (0.025 * LTQ[idxCL])
            C2 = ((1 - S + S * 10 ** (0.1 * (Le - LTQ[idxCL]))) ** 0.25) - 1
            coreLoudness[idxCL] = C1 * C2
            if coreLoudness[idxCL] <= 0:
                coreLoudness[idxCL] = 0
        criticalBandLevels[idxCL] = Le
    return coreLoudness


# Corrección del loudness específico de la banda más baja por silencio en esta banda
def f_corr_loudness(coreLoudness):
    CorrCL = 0.4 + 0.32 * np.float(np.power(coreLoudness[0], 0.2))
    if CorrCL < 1:
        coreLoudness[0] *= CorrCL
    return coreLoudness


# Cálculo del loudness específico con saltos Bark 0.1
def calc_slopes(coreLoudness):
    N1 = 0
    Z = 0.1
    Z1 = 0
    idxRNS = 0
    idxNS = 0
    Loudness = 0
    specLoudness = np.zeros(constantes['N_BARK_BANDS'])

    # Relación bandas tercio de octava y Bark
    ZUP = np.array([0.9, 1.8, 2.8, 3.5, 4.4, 5.4, 6.6, 7.9, 9.2, 10.6, 12.3, 13.8, 15.2, 16.7, 18.1, 19.3, 20.6, 21.8,
                    22.7, 23.6, 24.0])
    # Inclinación de la pendiente
    USL = np.array([[13.0, 8.2, 6.3, 5.5, 5.5, 5.5, 5.5, 5.5],
                    [9.0, 7.5, 6.0, 5.1, 4.5, 4.5, 4.5, 4.5],
                    [7.8, 6.7, 5.6, 4.9, 4.4, 3.9, 3.9, 3.9],
                    [6.2, 5.4, 4.6, 4.0, 3.5, 3.2, 3.2, 3.2],
                    [4.5, 3.8, 3.6, 3.2, 2.9, 2.7, 2.7, 2.7],
                    [3.7, 3.0, 2.8, 2.35, 2.2, 2.2, 2.2, 2.2],
                    [2.9, 2.3, 2.1, 1.9, 1.8, 1.7, 1.7, 1.7],
                    [2.4, 1.7, 1.5, 1.35, 1.3, 1.3, 1.3, 1.3],
                    [1.95, 1.45, 1.3, 1.15, 1.1, 1.1, 1.1, 1.1],
                    [1.5, 1.2, 0.94, 0.86, 0.82, 0.82, 0.82, 0.82],
                    [0.72, 0.67, 0.64, 0.63, 0.62, 0.62, 0.62, 0.62],
                    [0.59, 0.53, 0.51, 0.5, 0.42, 0.42, 0.42, 0.42],
                    [0.4, 0.33, 0.26, 0.24, 0.22, 0.22, 0.22, 0.22],
                    [0.27, 0.21, 0.2, 0.18, 0.17, 0.17, 0.17, 0.17],
                    [0.16, 0.15, 0.14, 0.12, 0.11, 0.11, 0.11, 0.11],
                    [0.12, 0.11, 0.1, 0.08, 0.08, 0.08, 0.08, 0.08],
                    [0.09, 0.08, 0.07, 0.06, 0.06, 0.06, 0.06, 0.05],
                    [0.06, 0.05, 0.03, 0.02, 0.02, 0.02, 0.02, 0.02]])
    # Rango del valor de la sonoridad
    RNS = np.array([21.5, 18.0, 15.1, 11.5, 9.0, 6.1, 4.4, 3.1, 2.13, 1.36, 0.82, 0.42, 0.3, 0.22, 0.15, 0.1, 0.035,
                    0.0])

    # Cálculo de sonoridad específica y total
    for idxCL in range(constantes['N_CORE_LOUDN']):
        idxCBN = idxCL - 1
        ZUP[idxCL] += 0.0001
        if idxCBN > constantes['N_CB_RANGES'] - 1:
            idxCBN = constantes['N_CB_RANGES'] - 1
        nextCriticalBand = 0
        while not nextCriticalBand:
            if N1 > coreLoudness[idxCL]:
                N2 = RNS[idxRNS]
                if N2 < coreLoudness[idxCL]:
                    N2 = coreLoudness[idxCL]
                DZ = (N1 - N2) / USL[idxRNS][idxCBN]
                Z2 = Z1 + DZ
                if Z2 > ZUP[idxCL]:
                    nextCriticalBand = 1
                    Z2 = ZUP[idxCL]
                    DZ = Z2 - Z1
                    N2 = N1 - DZ * USL[idxRNS][idxCBN]
                Loudness += DZ * (N1 + N2) / 2
                ZK = Z
                while ZK <= Z2:
                    specLoudness[idxNS] = N1 - (ZK - Z1) * USL[idxRNS][idxCBN]
                    idxNS += 1
                    ZK = ZK + 0.1
                Z = ZK

            else:
                if N1 < coreLoudness[idxCL]:
                    idxRNS = 0
                    while (idxRNS < constantes['N_RNS_RANGES']) and (RNS[idxRNS] >= coreLoudness[idxCL]):
                        idxRNS += 1
                nextCriticalBand = 1
                Z2 = ZUP[idxCL]
                N2 = coreLoudness[idxCL]
                Loudness += N2 * (Z2 - Z1)
                ZK = Z
                while ZK <= Z2:
                    specLoudness[idxNS] = N2
                    idxNS += 1
                    ZK = ZK + 0.1
                Z = ZK

            while (N2 <= RNS[idxRNS]) and (idxRNS < constantes['N_RNS_RANGES'] - 1):
                idxRNS += 1
            if idxRNS > constantes['N_RNS_RANGES'] - 1:
                idxRNS = constantes['N_RNS_RANGES'] - 1
            Z1 = Z2
            N1 = N2

        if Loudness < 0:
            Loudness = 0
    return Loudness, specLoudness


# Cálculo del loudness específico con saltos Bark 1
def calc_slopes_Bark(coreLoudness):
    N1 = 0
    Z = 1
    Z1 = 0
    idxRNS = 0
    idxNS = 0
    Loudness = 0
    specLoudness = np.zeros(variablesBark['N_BARK_BANDS'])

    # Relación bandas tercio de octava y Bark
    ZUP = np.array([0.9, 1.8, 2.8, 3.5, 4.4, 5.4, 6.6, 7.9, 9.2, 10.6, 12.3, 13.8, 15.2, 16.7, 18.1, 19.3, 20.6, 21.8,
                    22.7, 23.6, 24.0])
    # Inclinación de la pendiente
    USL = np.array([[13.0, 8.2, 6.3, 5.5, 5.5, 5.5, 5.5, 5.5],
                    [9.0, 7.5, 6.0, 5.1, 4.5, 4.5, 4.5, 4.5],
                    [7.8, 6.7, 5.6, 4.9, 4.4, 3.9, 3.9, 3.9],
                    [6.2, 5.4, 4.6, 4.0, 3.5, 3.2, 3.2, 3.2],
                    [4.5, 3.8, 3.6, 3.2, 2.9, 2.7, 2.7, 2.7],
                    [3.7, 3.0, 2.8, 2.35, 2.2, 2.2, 2.2, 2.2],
                    [2.9, 2.3, 2.1, 1.9, 1.8, 1.7, 1.7, 1.7],
                    [2.4, 1.7, 1.5, 1.35, 1.3, 1.3, 1.3, 1.3],
                    [1.95, 1.45, 1.3, 1.15, 1.1, 1.1, 1.1, 1.1],
                    [1.5, 1.2, 0.94, 0.86, 0.82, 0.82, 0.82, 0.82],
                    [0.72, 0.67, 0.64, 0.63, 0.62, 0.62, 0.62, 0.62],
                    [0.59, 0.53, 0.51, 0.5, 0.42, 0.42, 0.42, 0.42],
                    [0.4, 0.33, 0.26, 0.24, 0.22, 0.22, 0.22, 0.22],
                    [0.27, 0.21, 0.2, 0.18, 0.17, 0.17, 0.17, 0.17],
                    [0.16, 0.15, 0.14, 0.12, 0.11, 0.11, 0.11, 0.11],
                    [0.12, 0.11, 0.1, 0.08, 0.08, 0.08, 0.08, 0.08],
                    [0.09, 0.08, 0.07, 0.06, 0.06, 0.06, 0.06, 0.05],
                    [0.06, 0.05, 0.03, 0.02, 0.02, 0.02, 0.02, 0.02]])
    # Rango del valor de la sonoridad
    RNS = np.array([21.5, 18.0, 15.1, 11.5, 9.0, 6.1, 4.4, 3.1, 2.13, 1.36, 0.82, 0.42, 0.3, 0.22, 0.15, 0.1, 0.035,
                    0.0])

    # Cálculo de sonoridad específica y total
    for idxCL in range(constantes['N_CORE_LOUDN']):
        idxCBN = idxCL - 1
        ZUP[idxCL] += 0.0001
        if idxCBN > constantes['N_CB_RANGES'] - 1:
            idxCBN = constantes['N_CB_RANGES'] - 1
        nextCriticalBand = 0
        while not nextCriticalBand:
            if N1 > coreLoudness[idxCL]:
                N2 = RNS[idxRNS]
                if N2 < coreLoudness[idxCL]:
                    N2 = coreLoudness[idxCL]
                DZ = (N1 - N2) / USL[idxRNS][idxCBN]
                Z2 = Z1 + DZ
                if Z2 > ZUP[idxCL]:
                    nextCriticalBand = 1
                    Z2 = ZUP[idxCL]
                    DZ = Z2 - Z1
                    N2 = N1 - DZ * USL[idxRNS][idxCBN]
                Loudness += DZ * (N1 + N2) / 2
                ZK = Z
                while ZK <= Z2:
                    specLoudness[idxNS] = N1 - (ZK - Z1) * USL[idxRNS][idxCBN]
                    idxNS += 1
                    ZK = ZK + 1
                Z = ZK

            else:
                if N1 < coreLoudness[idxCL]:
                    idxRNS = 0
                    while (idxRNS < constantes['N_RNS_RANGES']) and (RNS[idxRNS] >= coreLoudness[idxCL]):
                        idxRNS += 1
                nextCriticalBand = 1
                Z2 = ZUP[idxCL]
                N2 = coreLoudness[idxCL]
                Loudness += N2 * (Z2 - Z1)
                ZK = Z
                while ZK <= Z2:
                    specLoudness[idxNS] = N2
                    idxNS += 1
                    ZK = ZK + 1
                Z = ZK

            while (N2 <= RNS[idxRNS]) and (idxRNS < constantes['N_RNS_RANGES'] - 1):
                idxRNS += 1
            if idxRNS > constantes['N_RNS_RANGES'] - 1:
                idxRNS = constantes['N_RNS_RANGES'] - 1
            Z1 = Z2
            N1 = N2

        if Loudness < 0:
            Loudness = 0
    return Loudness, specLoudness


# Conversión sonos a fonos
def sone2phon(Loudness):
    if Loudness >= 1:
        LN = 40 + 33.22 * np.log10(Loudness)
    else:
        LN = 40 * np.power(Loudness + 0.0005, 0.35)
    return LN

# Decaimiento temporal no lineal
def f_nl(coreLoudness):
    # Inicializacion variables
    Tvar = constantes['TVAR']
    Tshort = constantes['TSHORT']
    Tlong = constantes['TLONG']
    deltaT = 1 / (constantes['SR_LEVEL'] * constantes['NL_ITER'])
    P = (Tvar + Tlong) / (Tvar * Tshort)
    Q = 1 / (Tshort * Tvar)
    Lambda1 = -P / 2 + np.sqrt(P * P / 4 - Q)
    Lambda2 = -P / 2 - np.sqrt(P * P / 4 - Q)
    Den = Tvar * (Lambda1 - Lambda2)
    E1 = np.exp(Lambda1 * deltaT)
    E2 = np.exp(Lambda2 * deltaT)
    B = np.array([(E1 - E2) / Den,
                  ((Tvar * Lambda2 + 1) * E1 - (Tvar * Lambda1 + 1) * E2) / Den,
                  ((Tvar * Lambda1 + 1) * E1 - (Tvar * Lambda2 + 1) * E2) / Den,
                  (Tvar * Lambda1 + 1) * (Tvar * Lambda2 + 1) * (E1 - E2) / Den,
                  np.exp(-deltaT / Tlong),
                  np.exp(-deltaT / Tvar)])

    NlaLpDta = {
        'B': B,
        'UoLast': 0,
        'U2Last': 0
    }

    for idxCL in range(constantes['N_CORE_LOUDN']):
        NlaLpDta['UoLast'] = 0
        NlaLpDta['U2Last'] = 0

        for idxTime in range(np.shape(coreLoudness)[1] - 1):
            Delta = (coreLoudness[idxCL, idxTime + 1] - coreLoudness[idxCL, idxTime]) / float(constantes['NL_ITER'])
            Ui = coreLoudness[idxCL, idxTime]
            NlaLpDta = f_nl_lp(Ui, NlaLpDta)
            coreLoudness[idxCL, idxTime] = NlaLpDta['UoLast']
            Ui += Delta
            for idxI in range(constantes['NL_ITER'] - 1):
                NlaLpDta = f_nl_lp(Ui, NlaLpDta)
                Ui += Delta
        NlaLpDta = f_nl_lp(coreLoudness[idxCL, idxTime + 1], NlaLpDta) # Actualizar diccionario
        coreLoudness[idxCL, idxTime + 1] = NlaLpDta['UoLast']
    return coreLoudness


# Actualización parámetros para cálculo de decaimiento temporal
def f_nl_lp(Ui, NlaLpDta):
    if Ui < NlaLpDta['UoLast']:     # Caso 1
        if NlaLpDta['UoLast'] > NlaLpDta['U2Last']:     # Caso 1.1
            U2 = NlaLpDta['UoLast'] * NlaLpDta['B'][0] - NlaLpDta['U2Last'] * NlaLpDta['B'][1]
            Uo = NlaLpDta['UoLast'] * NlaLpDta['B'][2] - NlaLpDta['U2Last'] * NlaLpDta['B'][3]
            if Uo < Ui:     # Uo no puede ser menor que Ui
                Uo = Ui
            if U2 > Uo:     # U2 no puede ser mayor que Uo
                U2 = Uo
        else:       # Caso 1.2
            Uo = NlaLpDta['UoLast'] * NlaLpDta['B'][4]
            if Uo < Ui:     # Uo no puede ser menor que Ui
                Uo = Ui
            U2 = Uo
    else:
        if Ui == NlaLpDta['UoLast']:        # Caso 2
            Uo = Ui
            if Uo > NlaLpDta['U2Last']:     # Caso 2.1
                U2 = (NlaLpDta['U2Last'] - Ui) * NlaLpDta['B'][5] + Ui
            else:       # Caso 2.2
                U2 = Ui
        else:       # Caso 3
            Uo = Ui
            U2 = (NlaLpDta['U2Last'] - Ui) * NlaLpDta['B'][5] + Ui
    NlaLpDta['UoLast'] = Uo
    NlaLpDta['U2Last'] = U2
    return NlaLpDta


# Ponderación temporal
def loudness_zwicker_temporal_weighting(loudness):
    RATE = 2000
    Tau = 3.5 * 10**-3
    Filt1 = loudness_zwicker_lowpass_intp(loudness, 
                                          Tau, 
                                          RATE)
    Tau = 70 * 10**-3
    Filt2 = loudness_zwicker_lowpass_intp(loudness, 
                                          Tau, 
                                          RATE)
    Loudness = 0.47 * Filt1 + 0.53 * Filt2
    return Loudness


# Filtro paso bajo e interpolación para obtenr frecuancia de muetreo de 48 kHz
def loudness_zwicker_lowpass_intp(loudness, Tau, RATE=2000):
    numSamples = np.shape(loudness)[0]
    Y = 0
    Input = loudness.copy()
    Output = np.zeros(numSamples)

    A1 = np.exp(-1 / (RATE * constantes['LP_ITER'] * Tau))
    B0 = 1 - A1

    for idxTime in range(numSamples):
        Output[idxTime] = B0 * Input[idxTime] + A1 * Y
        Y = Output[idxTime]
        if idxTime < numSamples - 1:
            Xd = (Input[idxTime + 1] - Input[idxTime]) / constantes['LP_ITER']
            # Iteraciones internas / interpolacion
            for idxI in range(constantes['LP_ITER']):
                Input[idxTime] += Xd
                Y = B0 * Input[idxTime] + A1 * Y
    return Output


# Función principal sonidos estacionarios
def loudness_ISO532(ThirdOctaveLevels, SoundFieldDiffuse):
    ThirdOctaveIntens = f_corr_third_octave_intensities(ThirdOctaveLevels)
    LCB = f_calc_lcbs(ThirdOctaveIntens)
    coreLoudness = f_calc_core_loudness(ThirdOctaveLevels, 
                                        LCB, 
                                        SoundFieldDiffuse)
    coreLoudness = f_corr_loudness(coreLoudness)
    Loudness, specLoudness = calc_slopes(coreLoudness)
    LoudnessBark, specLoudnessBark = calc_slopes_Bark(coreLoudness)
    return Loudness, specLoudness, LoudnessBark, specLoudnessBark


# Función principal sonidos variantes en el tiempo
def loudness_ISO532_time(ThirdOctaveLevels, SoundFieldDiffuse, RATE=48000, CHUNK=4800):
    numSampleLevel = np.shape(ThirdOctaveLevels)[1]
    coreLoudness = np.zeros((21, numSampleLevel))
    for idxTime in range(numSampleLevel):
        ThirdOctaveIntens = f_corr_third_octave_intensities(ThirdOctaveLevels[:, idxTime])
        LCB = f_calc_lcbs(ThirdOctaveIntens)
        coreLoudness[:, idxTime] = f_calc_core_loudness(ThirdOctaveLevels[:, idxTime], 
                                                        LCB, 
                                                        SoundFieldDiffuse)
        coreLoudness[:, idxTime] = f_corr_loudness(coreLoudness[:, idxTime])
    coreLoudness = f_nl(coreLoudness)
    Loudness = np.zeros(numSampleLevel)
    specLoudness = np.zeros((constantes['N_BARK_BANDS'], 
                            numSampleLevel))
    for idxTime in range(numSampleLevel):
        Loudness[idxTime], specLoudness[:, idxTime] = calc_slopes(coreLoudness[:, idxTime])
    filtLoudness = loudness_zwicker_temporal_weighting(Loudness)
    return filtLoudness[::constantes['DEC_FACTOR']], specLoudness[:, ::constantes['DEC_FACTOR']]


# -*- coding: UTF-8 -*-
import sys

sys.path.append('../')
import librosa
import requests

import numpy as np
import datetime

SCORING_URL = 'https://dnsmos.azurewebsites.net/score-dec'
AUTH = ('netease',
        'decmos')


SCENARIOS = [
    'doubletalk_with_movement',
    'doubletalk',
    'farend_singletalk_with_movement',
    'farend_singletalk',
    'nearend_singletalk'
]


def read_and_process_audio_files(lpb_path, mic_path, enh_path,startPoint,SAMPLE_RATE):
    lpb_sig, _ = librosa.load(lpb_path, sr=SAMPLE_RATE)
    mic_sig, _ = librosa.load(mic_path, sr=SAMPLE_RATE)
    enh_sig, _ = librosa.load(enh_path, sr=SAMPLE_RATE)

    # Make the clips the same length
    min_len = np.min([len(lpb_sig), len(mic_sig), len(enh_sig)])
    lpb_sig = lpb_sig[:min_len]
    mic_sig = mic_sig[:min_len]
    enh_sig = enh_sig[:min_len]


    lpb_sig, mic_sig, enh_sig = process_audio(
        lpb_sig, mic_sig, enh_sig,startPoint,SAMPLE_RATE)

    return lpb_sig, mic_sig, enh_sig



def process_audio(lpb_sig, mic_sig, enh_sig,startPoint,SAMPLE_RATE):
    silence_duration = startPoint * SAMPLE_RATE
    lpb_sig = lpb_sig[silence_duration:]
    mic_sig = mic_sig[silence_duration:]
    enh_sig = enh_sig[silence_duration:]
    return lpb_sig, mic_sig, enh_sig


def get_score(lpb_sig, mic_sig, enh_sig, scenario):
    audio_data = {
        'lpb': lpb_sig.tolist(),
        'mic': mic_sig.tolist(),
        'enh': enh_sig.tolist(),
        'scenario': scenario
    }

    response = requests.post(SCORING_URL, json=audio_data, auth=AUTH)
    json_body = response.json()

    if 'error' in json_body:
        raise Exception(json_body['error'])

    return json_body


def cal_aec_mos(refFile=None,micFile=None,testFile=None,scenario=1,startPoint=0,SAMPLE_RATE=48000):
    """
    Parameters
    ----------
    refFile
    micFile
    testFile
    scenario  0: doubletalk_with_movement 1:doubletalk 2:farend_singletalk_with_movement 3:farend_singletalk 4:nearend_singletalk
    startPoint： determine from which audio sample (startPoint(in second) * SAMPLE_RATE) to calculte
    SAMPLE_RATE：
    Returns
    -------

    """
    start = datetime.datetime.now()
    lpb_sig, mic_sig, enh_sig = read_and_process_audio_files(
        refFile, micFile, testFile,startPoint,SAMPLE_RATE)
    sjson = get_score(lpb_sig,mic_sig,enh_sig,SCENARIOS[scenario])
    end = datetime.datetime.now()
    print('time duration:',end-start)
    return sjson

if __name__ == '__main__':
    path = r'D:\AudioPublicWork\3a_auto_test_porject\3a_auto_test_porject\08_TestDstFiles\sdk_zego_vivo_y3hf_music_V_shengbo_compare\aec\Speech\TestCase_01_None_None\near_cn'
    ref = path + '\\' + 'far_cn_minus_30db.wav'
    mic =  path + '\\' + 'stdRefFile.wav'
    test =  path + '\\' + 'mixDstFile.wav'
    cal_aec_mos(testFile=test,refFile=ref,micFile=mic,startPoint=0,SAMPLE_RATE=16000,scenario=0)



# -*- coding: utf-8 -*-
# Andrea Castiella Aguirrezabala
# Diseño filtro ponderado A
# Definición de filtro ponderado A segun IEC/CD 1672

from numpy import pi, convolve
from scipy import signal
from scipy.signal import lfilter

# Obtener coeficientes filtro ponderación A
def coeficientesPonderacionA(RATE=48000):
    f1 = 20.598997
    f2 = 107.65265
    f3 = 737.86223
    f4 = 12194.217
    A1000 = 1.9997

    NUMs = [(2*pi*f4)**2*(10**(A1000/20)), 0, 0, 0, 0]
    DENs = convolve([1., +4*pi * f4, (2*pi * f4)**2],
        [1., +4*pi * f1, (2*pi * f1)**2])
    DENs = convolve(convolve(DENs, [1., 2*pi * f3]),
        [1., 2*pi * f2])

    # Uso de una transformación bilineal para obtener el filtro digital
    return signal.bilinear(NUMs, DENs, RATE)


# Filtrar señal para ponderación A.
def filtroPonderacionA(data, RATE=48000):
    b, a = coeficientesPonderacionA(RATE)
    pond_A = lfilter(b, a, data)
    return pond_A


# -*- coding: utf-8 -*-
# Andrea Castiella Aguirrezabala
# Fluctuation Strength Zwicker y Fastl "Psychoacoustics: Facts and Models"

from algorithmLib.Zwicker_loudness.loudness_ISO532 import sone2phon
import numpy as np
from scipy import signal

# Cálculo de FS
def acousticFluctuation(specificLoudness, fmod=4):
    specificLoudnessdiff = np.zeros(len(specificLoudness))
    for i in range(len(specificLoudness)):
        if i == 0:
            specificLoudnessdiff[i] = specificLoudness[i]
        else:
            specificLoudnessdiff[i] = abs(specificLoudness[i] - specificLoudness[i-1])
    F = (0.008*sum(0.1*specificLoudnessdiff))/((fmod/4)+(fmod/4))
    return F


# Detección de la frecuencia de modulación
def fmoddetection(specificLoudness, fmin=.2, fmax=64):
    if len(specificLoudness.shape) == 1:
        specificLoudness = np.reshape(specificLoudness, (1, len(specificLoudness)))
    else:
        specificLoudness = np.transpose(specificLoudness)
    phon = np.zeros((specificLoudness.shape[0], specificLoudness.shape[1], 1))
    for i, ms in enumerate(specificLoudness):
        for j, bark in enumerate(ms):
            phon[i, j, 0] = sone2phon(bark)
    FSpec = 1/0.002

    # Reagrupar saluda en bandas de 24 o 47 Bark
    phon1024a = np.reshape(phon,  newshape = (24, phon.shape[0], 10))
    # Overlap
    phon1024b = np.reshape(phon[:, 5:235],  newshape = ( 23, phon.shape[0], 10)) # Deja fuera los primeros 5 y los últimos 5
    h = np.hamming(10)
    phonB = np.zeros((phon.shape[0], 47))
    phonBtempa = np.sum((phon1024a * h), 2)
    phonB[:,0:47:2] = np.transpose(phonBtempa)
    phonBtempb = np.sum((phon1024b * h), 2)
    phonB[:,1:46:2] = np.transpose(phonBtempb)
    
    phonBm = phonB - (np.mean(phonB, 0))    # Eliminar media
    phonBm = np.maximum(0, phonBm)  # Eliminar parte negativa
    pbfstd = 5*(np.std(phonB, 0, ddof=1))   # Recortar amplitudes extremas
    phonBm = np.minimum(phonBm, pbfstd)
    phonBf = hpfilt(phonBm) # Eliminar bajas frecuencias
    pbfstd2 = np.std(phonBf, 0, ddof=1) # Recortar amplitudes extremas
    phonBf = np.minimum(phonBf, pbfstd2)
    phonBf = np.maximum(phonBf, -pbfstd2)
    NP = np.maximum(8192, 2**(next_power_of_2(specificLoudness.shape[0])))
    if not NP <= np.maximum(8192, 2*specificLoudness.shape[0]):
        print('Error')
        return None
    fmin = np.int(np.floor(NP*fmin/FSpec))    # Frecuncia de detección mínima
    fmax = np.int(np.ceil(NP*fmax/FSpec))    # Frecuencia de detección máxima
    X = np.sum(np.abs(np.fft.fft(phonBf,np.int(NP), axis=0)), 1)
    if not X.shape[0] <= NP:
        print('Error')
        return None
    nf = fmax-fmin+1
    t = np.arange(fmin,fmax+1)
    b = np.matmul(np.linalg.pinv(np.stack((np.ones(nf), t), axis=1)), np.log(np.maximum(X[fmin-1:fmax], np.finfo(float).tiny))) # Evitar log(0)
    XR = np.exp(b[1]*t+b[0])
    idx = np.argmax(X[fmin-1:fmax]- XR)
    idx = idx + fmin - 1
    cf = (idx)*FSpec/(NP-1)
    if idx > 4:                                 
        cf = refineCF(cf, X[idx-3:idx+2], NP, FSpec)
    return cf


# Cálculo potencia de dos inmediata
def next_power_of_2(x):
    return 1 if x == 0 else np.log2(2**(x - 1).bit_length())


# Filtro paso alto
def hpfilt(x):
    sos = np.array([
        [.998195566288485491845960950741, -1.996391132576970983691921901482, .998195566288485491845960950741, 1, -1.994897079594937228108619819977, .994904703139997681482498137484],
        [.998195566288485491845960950741, -1.996391132576970983691921901482, .998195566288485491845960950741, 1, -1.997878669564554510174048118643, .997886304503829646428414434922]])
    fn = 10**4
    xL = np.vstack((x, np.zeros((fn, x.shape[1]))))
    xLf = np.flipud(signal.sosfilt(sos, np.flipud(signal.sosfilt(sos, xL))))
    return xLf[0:x.shape[0], :]


# refinar frecuencia detectada
def refineCF(cf, p, NP, FSpec):
    rd_denom = p[0]+p[4]-16*(p[1]+p[3])+30*p[2]
    if not rd_denom == 0:
        rd = (p[0]-p[4]+8*(p[3]-p[1])) / rd_denom
        if np.abs(rd) < .5:
            cf = cf + rd*FSpec/(NP-1)
        else:
            p = p[1:4]
            if p[0]-2*p[1]+p[2] < 0:
                rd_denom = 2*(p[0]+p[2]-2*p[1])
                if not rd_denom == 0:
                    rd = (3*p[0]-4*p[1]+p[2])
                    if np.abs(rd) < .5:
                        cf = cf + rd*FSpec/(NP-1)
    return cf


# -*- coding: UTF-8 -*-
import sys

sys.path.append('../')
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_
import ctypes
import platform
#  * -------------------------------------------------------------------------
#  * Arguments    : const emxArray_real_T *sig_mic
#  *                const emxArray_real_T *sig_far
#  *                const emxArray_real_T *sig_ref
#  *                double fs_mic
#  *                double fs_far
#  *                double type
#  *                double *ERLE
#  *                double *output_std
#  *                double *residual_avgdB
#  *                double *err
#  * Return Type  : void
#  */
# void ERLE_estimation(const emxArray_real_T *sig_mic, const emxArray_real_T
#                      *sig_far, const emxArray_real_T *sig_ref, double fs_mic,
#                      double fs_far, double type, double *ERLE, double
#                      *output_std, double *residual_avgdB, double *err)

def cal_erle(micFile = None,testFile =None, refFile =None,targetType=0):

    """
    %         type- input signal type:
    %               0:Chiness
    %               1:English
    %               2:Single Digit
    %               3:Music
    Parameters
    ----------
    inFile
    output
    refFile
    targetType

    Returns
    -------

    """
    instruct,insamplerate,_ = get_data_of_ctypes_(micFile,True)
    teststruct,outsamplerate,_ = get_data_of_ctypes_(testFile,True)
    refstruct, refsamplerate, _ = get_data_of_ctypes_(refFile,True)

    # if refsamplerate != testsamplerate :
    #     raise TypeError('Different format of ref and test files!')
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/ERLE_estimation.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/ERLE_estimation.dylib')

    mydll.ERLE_estimation.argtypes = [POINTER(emxArray_real_T),POINTER(emxArray_real_T),POINTER(emxArray_real_T),c_double,c_double,c_double,POINTER(c_double),POINTER(c_double),POINTER(c_double),POINTER(c_double)]
    data_format = c_double*11
    gain_table = data_format()
    DR = data_format()
    ERLE,output_std,err,residual_avgdB = c_double(0.0),c_double(0.0),c_double(0.0),c_double(0.0)
    mydll.ERLE_estimation(byref(instruct),byref(teststruct),byref(refstruct),c_double(insamplerate),c_double(outsamplerate),c_double(targetType),byref(ERLE),byref(output_std),byref(residual_avgdB),byref(err))

    if err.value == 0.0:
        return ERLE.value,output_std.value,residual_avgdB.value
    else:
        return None,None,None


if __name__ == '__main__':
    import platform
    print(platform.platform().split('-')[0])
    # micfile = r'C:\Users\vcloud_avl\Documents\我的POPO\0\stdRefFile.wav'
    # test = r'C:\Users\vcloud_avl\Documents\我的POPO\0\mixDstFile.wav'
    # ref = R'C:\Users\vcloud_avl\Documents\我的POPO\0\ref_cn.wav'
    micfile = r'D:\0\stdRefFile.wav'
    test = r'D:\0\mixDstFile.wav'
    ref = r'D:\0\ref_en.wav'
    ERLE,output_std,residual_avgdB = cal_erle(micFile=micfile,testFile=test,refFile=ref,targetType=0)
    print('ERLE:{},output_std:{},residual_avgdB:{}'.format(ERLE,output_std,residual_avgdB))



# -*- coding: utf-8 -*-
import sys
sys.path.append('./')
sys.path.append('../')

from commFunction import get_rms,make_out_file,get_ave_rms
import numpy as np

from SNR_ESTIMATION.MATCH_SIG import match_sig
from commFunction import get_data_array


speechSection = [12, 15]
noiseSection = [0, 10]
FRAME_LEN = 960



def get_data_pairs(srcFile=None,testFile=None):
    """
    Parameters
    ----------
    srcFile
    testFile
    Returns
    -------
    """

    samples = int(match_sig(refFile=srcFile, testFile=testFile))

    dataSrc, fs, chn = get_data_array(srcFile)
    dataTest, fs2, chn2 = get_data_array(testFile)

    print(dataTest,dataSrc,samples)
    assert fs == fs2
    assert  chn2 == chn
    assert samples > 0

    dataTest = dataTest[samples:]
    M,N = len(dataSrc),len(dataTest)
    targetLen = min(M,N)
    return dataSrc[:targetLen],dataTest[:targetLen],fs,chn


def cal_noise_converge(dataSrc,dataTest,fs,chn):
    """
    Parameters
    ----------
    dataSrc
    dataTest
    Returns
    -------
    """
    srcSpeechLevel = get_rms(dataSrc[fs*speechSection[0]:fs*speechSection[1]])
    curSpeechLevel = get_rms(dataTest[fs*speechSection[0]:fs*speechSection[1]])

    # log（V1 / V2) = X/20

    gain = np.power(10,(srcSpeechLevel - curSpeechLevel)/20)
    newData = dataTest.astype(np.float32) * gain
    make_out_file('source.wav', dataSrc.astype(np.int16), fs, chn)
    make_out_file('target.wav',newData.astype(np.int16),fs,chn)

    n_sengen = len(newData) // FRAME_LEN
    MAX_RMS = -120
    for a in range(n_sengen):
        curLevel = get_rms(newData[a*FRAME_LEN:(a+1)*FRAME_LEN])
        print(MAX_RMS,curLevel)
        if curLevel > MAX_RMS:
            MAX_RMS = curLevel
        if curLevel < MAX_RMS - 20:
            break
    converge = a * FRAME_LEN / fs

    nsLevel = get_ave_rms(dataSrc[int(converge * fs) :noiseSection[1]* fs]) - get_ave_rms(newData[int(converge * fs) :noiseSection[1]* fs])
    return converge, nsLevel
    #TODO 收敛时间
    #TODO 降噪量



def cal_noise_Supp(srcFile,testFile):
    """
    Parameters
    ----------
    data
    Returns
    -------
    """
    srcdata, dstdata, fs, chn = get_data_pairs(srcFile=srcFile, testFile=testFile)
    return  cal_noise_converge(srcdata,dstdata,fs,chn)





if __name__ == '__main__':
    src = 'car_noise_speech.wav'
    dst = 'speech_cn.wav'
    dur = cal_noise_Supp(src,dst)
    print(dur)
    pass


from operator import methodcaller
from computeAudioQuality.mainProcess import computeAudioQuality
from ctypes import  *


def compute_audio_quality(metrics, testFile=None, refFile=None,micFile=None,cleFile=None, aecCaliFile=None,outFile=None, noiseFile=None,
                              samplerate=16000, bitwidth=2, channel=1, refOffset=0, testOffset=0, maxComNLevel=-48.0,
                              speechPauseLevel=-35.0,audioType=0,aecStartPoint=0,aecTargetType=0,aecScenario=0,rmsCalsection=None):
    """
    :param metrics: G160/P563/POLQA/PESQ/STOI/STI/PEAQ/SDR/SII/LOUDNESS/MUSIC/MATCH/
                    TRANSIENT/GAINTABLE/ATTACKRELEASE/MUSICSTA/AGCDELAY/MATCHAEC/
                    ELRE/SLIENCE/FORMAT/AECMOS/AIMOS/TRMS/ARMS/NOISE，必选项
    # G160 无采样率限制；  WAV/PCM输入 ；三端输入: clean、ref、test；无时间长度要求；
    # P563 8000hz(其他采样率会强制转换到8khz)；  WAV/PCM输入 ；单端输入: test；时长 < 20s；
    # POLQA 窄带模式  8k  超宽带模式 48k ；WAV/PCM输入 ；双端输入：ref、test；时长 < 20s；
    # PESQ 窄带模式  8k   宽带模式 16k ；WAV/PCM输入 ；双端输入：ref、test；时长 < 20s；
    # STOI 无采样率限制; 双端输入：ref、test；无时间长度要求；
    # STI >8k(实际会计算8khz的频谱)； WAV/PCM输入 ；双端输入：ref、test；时长 > 20s
    # PEAQ 无采样率限制；WAV/PCM输入 ；双端输入：ref、test；无时间长度要求；
    # SDR 无采样率限制; WAV/PCM输入 ; 双端输入：ref、test；无时间长度要求；
    # MATCH 无采样率限制; WAV/PCM输入;三端输入：ref、test、out； 无时间长度要求；
    # MUSIC 无采样率限制;WAV/PCM输入;双端输入：ref、test；无时间长度要求；
    # TRANSIENT 无采样率限制,WAV/PCM输入;三端输入：cle、noise、test； 无时间长度要求；
    # GAINTABLE 无采样率限制,WAV/PCM输入;双端输入：ref、test；固定信号输入；
    # ATTACKRELEASE 无采样率限制,WAV/PCM输入;双端输入：ref、test；固定信号输入；
    # MUSICSTA 无采样率限制,WAV/PCM输入;双端输入：ref、test；无时间长度要求；
    # AGCDELAY 无采样率限制,WAV/PCM输入;双端输入：ref、test；无时间长度要求；
    # MATCHAEC 无采样率限制 WAV/PCM输入;三端输入：ref、mic,test，；无时间长度要求；
    # ELRE 无采样率限制 WAV/PCM输入;三端输入：mic,ref、test；无时间长度要求；
    # SLIENCE 无采样率限制 WAV/PCM/MP4输入;单端输入：test；无时间长度要求；
    # FORMAT 无采样率限制 WAV/MP4输入;单端输入：test；无时间长度要求；
    # AECMOS 无采样率限制 WAV/PCM输入 ；三端输入：mic,ref、test；无时间长度要求；
    # AIMOS 无采样率限制 WAV/PCM输入 ；单端输入：test；无时间长度要求；
    # TRMS 无采样率限制 WAV/PCM输入 ；单端输入：test；无时间长度要求；
    # ARMS 无采样率限制 WAV/PCM输入 ；单端输入：test；无时间长度要求；
    # NOISE 无采样率限制 WAV/PCM输入 ；双端输入：ref、test；无时间长度要求；
    不同指标输入有不同的采样率要求，如果传入的文件不符合该指标的要求，会自动变采样到合法的区间
    :param testFile: 被测文件，必选项
    :param refFile:  参考文件，可选项，全参考指标必选，比如POLQA/PESQ/PEAQ
    :param micFile:  micIN，可选项，回声指标必选，MATCHAEC/ELRE/AECMOS
    :param cleFile:  干净语音文件，可选项，G160,TRANSIENT需要
    :param noiseFile 噪声文件，可选项，突发噪声信噪比计算需要
    :param aecCaliFile 用于做AEC对齐的校准文件  MATCHAEC专用
    :param outFile 输出文件，可选项，对齐文件可选
    :param samplerate: 采样率，可选项，pcm文件需要 default = 16000
    :param bitwidth: 比特位宽度，可选项，pcm文件需要 default = 2
    :param channel: 通道数，可选项，pcm文件需要 default = 1
    :param refOffset: ref文件的样点偏移，可选项，指标G160需要
    :param testOffset: test文件的样点偏移，可选项，指标G160需要
    :param maxComNLevel: 测试G160文件的最大舒适噪声
    :param speechPauseLevel 测试G160文件的语音间歇段的噪声
    :param audioType  输入音频的模式 0：语音 1：音乐 MATCH/GAINTABLE需要
    :param aecStartPoint  计算AECMOS，选择从第几秒开始计算
    :param aecTargetType  0:Chiness 1:English 2:Single Digit 3:Music  计算MATCHAEC/ELRE
    :param aecScenario 计算aec mos专用     0:'doubletalk_with_movement', 1:'doubletalk', 2:'farend_singletalk_with_movement', 3:'farend_singletalk', 4:'nearend_singletalk'
    :param rmsCalsection 计算rms的区间 TRMS和ARMS需要，时间单位s，比如：[1,20]
    :return:
    """
    paraDicts = {
        'metrics':metrics,
        'testFile':testFile,
        'refFile':refFile,
        'micFile':micFile,
        'cleFile':cleFile,
        'noiseFile':noiseFile,
        'aecCaliFile':aecCaliFile,
        'outFile':outFile,
        'samplerate':samplerate,
        'bitwidth':bitwidth,
        'channel':channel,
        'refOffset':refOffset,
        'testOffset':testOffset,
        'maxComNLevel':maxComNLevel,
        "speechPauseLevel":speechPauseLevel,
        "audioType":audioType,
        "aecStartPoint":aecStartPoint,
        "aecTargetType":aecTargetType,
        'aecScenario':aecScenario,
        'rmsCalsection':rmsCalsection
    }
    comAuQUA = computeAudioQuality(**paraDicts)
    return methodcaller(metrics)(comAuQUA)

if __name__ == '__main__':

    # speech = r'D:\AutoWork\audiotestalgorithm\algorithmLib\SNR_ESTIMATION\speech.wav'
    # music = r'D:\AutoWork\audiotestalgorithm\algorithmLib\SNR_ESTIMATION\music_rap.wav'
    # transi = r'D:\AutoWork\audiotestalgorithm\algorithmLib\SNR_ESTIMATION\transientNoise.wav'
    # test = r'D:\AutoWork\audiotestalgorithm\algorithmLib\SNR_ESTIMATION\test.wav'
    # res = compute_audio_quality('MUSIC',refFile=speech,testFile=music)
    #
    # print(res)
    #
    # res = compute_audio_quality('TRANSIENT',cleFile=speech,noiseFile=transi,testFile=test)
    # print(res)
    #
    # res = compute_audio_quality('MATCH',refFile=speech,testFile=test,outFile='123.wav')
    # print(res)
    #print(compute_audio_quality('G160', testFile=src,refFile=src,samplerate=16000))

    #print(match_sig(refFile='speech.wav', targetFile='test.wav', outFile='outfile.wav'))
    import time
    for a in range(200):
        time.sleep(1)
        src = r'E:\audioalgorithm\audiotestalgorithm\demos\02_p563_demo\cleDstFile.wav'
        print(compute_audio_quality('P563',testFile=src))
    exit(0)
    file = r'C:\Users\vcloud_avl\Downloads\agc_eva\speech_attackrelease.wav'
    test = r'C:\Users\vcloud_avl\Downloads\agc_eva\test_attackrelease.wav'
    print(compute_audio_quality('ATTACKRELEASE',refFile=file,testFile=test))

    file = r'C:\Users\vcloud_avl\Downloads\agc_eva\speech_gaintable.wav'
    test = r'C:\Users\vcloud_avl\Downloads\agc_eva\test.wav'
    lim,gain_table,DR = compute_audio_quality('GAINTABLE',refFile=file,testFile=test,audioType=1)
    print(lim,gain_table[0],DR[2])
    for a in gain_table:
        print(a)
    for a in DR:
        print(a)

    file = r'C:\Users\vcloud_avl\Downloads\agc_eva\music_stability_.wav'
    test = r'C:\Users\vcloud_avl\Downloads\agc_eva\test_music_stability.wav'
    res = compute_audio_quality('MUSICSTA',refFile=file, testFile=test)
    for a in res:
        print(a)

    file = r'C:\Users\vcloud_avl\Downloads\agc_eva\speech_gaintable.wav'
    test = r'C:\Users\vcloud_avl\Downloads\agc_eva\test.wav'
    delay = compute_audio_quality('AGCDELAY',refFile=file,testFile=test)
    print(delay)

    compute_audio_quality('MATCH',refFile=file,testFile=test,outFile='out.wav',audioType=1)


import os
import sys


def cal_peaq(ref, est):
    '''
    :param ref:
    :param est:
    :return:
    '''
    cmd = sys.prefix + '/peaqb.exe -r '+ ref + ' -t ' + est
    result = os.popen(cmd)
    res = result.read()
    for line in res.splitlines():
        print(line)

if __name__ == '__main__':
    cal_peaq('eval1.wav','eval1_echo100.wav')


# -*- coding: utf-8 -*-
# Andrea Castiella Aguirrezabala
# Sharpness DIN 45692

import numpy as np

def calc_sharpness(loudness, specificLoudness):
	# Evitar división entre 0
    if loudness == 0:
        loudness = 1e-8
    n = len(specificLoudness)

    # Zwicker method
    gz_Z = np.ones(n)
    for z in range(158, n):
        gz_Z[z] = 0.15*np.e**(0.42 * ((z+1)/10-15.8))+0.85
    suma = 0
    for i, z in enumerate(np.arange(0.1, n / 10 + 0.1, 0.1)):
        suma += specificLoudness[i] * gz_Z[i] * round(z, 1) * 0.1
    sharpnessZwicker = 0.11 * suma / loudness

    # Von Bismarck method
    gz_VB = np.ones(n)
    for z in range(150, n):
        gz_VB[z] = 0.2 * np.e ** (0.308 * ((z+1)/10 - 15)) + 0.8
    suma = 0
    for i, z in enumerate(np.arange(0.1, n / 10 + 0.1, 0.1)):
        suma += specificLoudness[i] * gz_VB[i] * round(z, 1) * 0.1
    sharpnessVonBismarck = 0.11 * suma / loudness

    # Aures method
    gz_A = np.ones(n)
    for z in range(0, n):
    	zi = np.float((np.float(z+1))/np.float(10))
        gz_A[z] = (0.078*np.e**(0.171*(zi))/(zi))*(loudness/np.log(0.05*loudness+1))
    suma = 0
    for i, z in enumerate(np.arange(0.1, n / 10 + 0.1, 0.1)):
        suma += specificLoudness[i] * gz_A[i] * round(z, 1) * 0.1
    sharpnessAures = 0.11 * suma / loudness

    return round(sharpnessZwicker, 2), round(sharpnessVonBismarck, 2), round(sharpnessAures, 2)



# -*- coding: utf-8 -*-
# Andrea Castiella Aguirrezabala
# Main

import math
import numpy as np
import pyaudio
import loudness_ISO532
import sys
import ThirdOctaveFilters as TOF
import Sharpness
from FiltroPonderacionA import filtroPonderacionA
from Roughness import acousticRoughness
from FluctuationStrength import acousticFluctuation, fmoddetection
from PsychoacousticAnnoyance import psychoacousticAnnoyance

# Constantes e inicialización pyaudio
CHUNK = 4800  # Tamaño en muestras almacenadas en cada array
RATE = 48000  # Muestras por segundo
TimeVarying = False
p = pyaudio.PyAudio()
stream = p.open(format=pyaudio.paFloat32, 
					rate=RATE, 
					channels=1, 
					input=True, 
					input_device_index=2,
					frames_per_buffer=CHUNK)  
					# Input Device = 0 si ordenador, 1 si cascos, 2 si Raspberry

# Funciones principales
def mainEstacionario(data, RATE, CHUNK):
    # Filtro tercio de octava
    ThirdOctave = TOF.ThirdOctaveBandFilter(frame=data, CHUNK=CHUNK)
    ThirdOctaveSPL = TOF.ThirdOctaveSPL(ThirdOctaveBands=ThirdOctave, 
                                        CHUNK=CHUNK, 
                                        RATE=RATE, 
                                        TimeSkip=0)
    # Loudness
    loudness, specLoudness, _, _ = loudness_ISO532.loudness_ISO532(ThirdOctaveSPL,
                                                                   SoundFieldDiffuse=0)
    loudnessPhon = loudness_ISO532.sone2phon(loudness)
    print('Loudness total en sonios: ', round(loudness, 1), ' sonios')
    print('Loudness total en fonos', round(loudnessPhon, 2), ' fonos')

    # Cálculo sharpness
    sharpnessZwicker, sharpnessVonBismarck, sharpnessAures = Sharpness.calc_sharpness(loudness, 
                                                                                      specLoudness)
    print('Sharpness Zwicker: ', sharpnessZwicker, ' acum')
    print('Sharpness VB: ', sharpnessVonBismarck, ' acum')
    print('Sharpness Aures: ', sharpnessAures, ' acum')

    
    specLoudness2 = np.stack((specLoudness, specLoudness), axis=1)
    fmodFS = fmoddetection(specLoudness2, fmin=.2, fmax=64)
    #Fluctuation strength
    FS = acousticFluctuation(specLoudness, fmodFS)
    fmodR = fmoddetection(specLoudness2, fmin=40, fmax=150)
    # Roughness
    R = acousticRoughness(specLoudness, fmodR)
    # Psychoacoustic Annoyance
    PA = psychoacousticAnnoyance(specLoudness, sharpnessZwicker, FS, R)

    print('Fluctuation strength: ', round(FS, 2), ' vacil')
    print('Roughness: ', round(R, 2), ' asper')
    print('Psychoacoustic annoyance: ', round(PA, 2))

def mainVarianteTiempo(data, RATE, CHUNK):
    # Filtrado tercio de octava
    ThirdOctaveLevelTime, _, _ = TOF.ThirdOctaveLevelTime(data)
    # Loudness
    loudness, specLoudness = loudness_ISO532.loudness_ISO532_time(ThirdOctaveLevelTime, 
                                                                  SoundFieldDiffuse=0)

    loudnessPhon = []
    for loundness_i in loudness:
        loudnessPhon.append(loudness_ISO532.sone2phon(loundness_i))
    
    loudnessPhon = [round(num, 1) for num in loudnessPhon]

    print('Loudness total en sonios: ', loudness, ' sonios')
    print('Loudness total en fonos', loudnessPhon, ' fonos')

    sharpnessZwicker = []
    sharpnessVonBismarck = []
    sharpnessAures = []

    for i in range(len(loudness)):
        sharpnessZwickerTemp, sharpnessVonBismarckTemp, sharpnessAuresTemp = Sharpness.calc_sharpness(loudness[i], 
                                                                                                      specLoudness[:, i])
        sharpnessZwicker.append(sharpnessZwickerTemp)
        sharpnessVonBismarck.append(sharpnessVonBismarckTemp)
        sharpnessAures.append(sharpnessAuresTemp)

    print('Sharpness Zwicker: ', sharpnessZwicker)
    print('Sharpness VB: ', sharpnessVonBismarck)
    print('Sharpness Aures: ', sharpnessAures)

    # Detección frecuencia moduladora
    fmodFS = fmoddetection(specLoudness, fmin=.2, fmax=64)
    fmodR = fmoddetection(specLoudness, fmin=40, fmax=150)

    FS = []
    R = []
    PA = []

    for i in range(len(loudness)):
        #Fluctuation strength
        FS.append(acousticFluctuation(specLoudness[:, i], fmodFS))
        # Roughness
        R.append(acousticFluctuation(specLoudness[:, i], fmodR))
        # Psychoacoustic Annoyance
        PA.append(psychoacousticAnnoyance(specLoudness[:, i], sharpnessZwicker[i], FS[i], R[i]))

    FS = [round(num, 4) for num in FS]
    R = [round(num, 4) for num in R]
    PA = [round(num, 4) for num in PA]

    print('Fluctuation strength: ', FS, ' vacil')
    print('Roughness: ', R, ' asper')
    print('Psychoacoustic annoyance: ', PA)

def captarAudio(corr_factor = 4.73):
    # Almacenar audio en array
    data = np.frombuffer(stream.read(CHUNK, 
                         exception_on_overflow=False), 
                         dtype=np.float32)
    # Factor de corrección del dispositivo que está siendo utilizado para captar la señal. 94 dB a 1 kHz 1 Pa de referencia.
    data = data * corr_factor
    return data

def calcSPL_SPLA(data):
    # Cálculo del valor rms de la señal captada (CHUNK)
    rms_corr = np.sqrt(np.mean(np.absolute(data.astype(float)) ** 2))
    # Cálculo del valor SPL
    SPL = 20 * math.log(rms_corr / (20 * (10 ** -6)), 10)
    print("Z-weighted", round(SPL,1))

    # Fitlrado ponderación A
    pond_A = filtroPonderacionA(data, RATE)
    # Cálculo del valor rms de la señal ponderada A y corrección
    rms_A = np.sqrt(np.mean(np.absolute(pond_A.astype(float)) ** 2))
    rms_A_corr = rms_A
    # Cálculo del valor SPL ponderado A
    SPL_A = 20 * math.log(rms_A_corr / (20 * (10 ** -6)), 10)
    print("A-weighted", round(SPL_A,1))

try:
    # Bucle para captar audio de forma continua
    while True:  
        # Captación audio
        data = captarAudio()

        # Cálculo SPL y SPL(A)
        calcSPL_SPLA(data)

        # Estacionario
        if not TimeVarying:
            mainEstacionario(data, RATE, CHUNK)

        # Variante en el tiempo
        if TimeVarying:
            mainVarianteTiempo(data, RATE, CHUNK)

# Salir del bucle con ctr+c
except KeyboardInterrupt:
    # Cierre stream
    stream.stop_stream()
    stream.close()
    p.terminate
    print('Interrupción de teclado. Finalizando programa.')
    sys.exit()





# -*- coding: UTF-8 -*-
import sys

sys.path.append('../')
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_
import ctypes
import platform
#  * -------------------------------------------------------------------------
#  * Arguments    : const emxArray_real_T *ref_in
#  *                const emxArray_real_T *sig_in
#  *                double fs
#  *                double *delay
#  *                double *err
#  * Return Type  : void
#  */
# void delay_estimation_15s(const emxArray_real_T *ref_in, const emxArray_real_T
#   *sig_in, double fs, double *delay, double *err)

def cal_DELAY(refFile=None, testFile=None):
    """
    """
    refstruct,refsamplerate,_ = get_data_of_ctypes_(refFile)
    teststruct,testsamplerate,_ = get_data_of_ctypes_(testFile)

    if refsamplerate != testsamplerate :
        raise TypeError('Different format of ref and test files!')

    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/agcDelay.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/agcDelay.dylib')

    mydll.delay_estimation_15s.argtypes = [POINTER(emxArray_real_T),POINTER(emxArray_real_T),c_double,POINTER(c_double),POINTER(c_double)]
    delay,err = c_double(0.0),c_double(0.0)
    mydll.delay_estimation_15s(byref(refstruct),byref(teststruct),c_double(refsamplerate),byref(delay),byref(err))

    if err.value == 0.0:
        return delay.value
    else:
        return None


if __name__ == '__main__':
    file = r'C:\Users\vcloud_avl\Downloads\agc_eva\speech_gaintable.wav'
    test = r'C:\Users\vcloud_avl\Downloads\agc_eva\test.wav'
    delay = cal_DELAY(refFile=file,testFile=test)
    print(delay)
    pass


# -*- coding: utf-8 -*-
# Andrea Castiella Aguirrezabala
# Algoritmo para la detección de voz

import numpy as np
import statistics
import pyaudio

# Detección de voz
def VAD(frames):
    speech = None
    Energy_PrimThresh = 40
    F_PrimThresh = 185
    SF_PrimThresh = 5
    silenceCount = 0
    speechCount = 0

    for frame in frames:
        Ei = shortTermEnergy(frame)
        Fi = dominantFrequency(frame)
        SFMi = SFM(frame)

        E_min = np.min(Ei)
        F_min = np.min(Fi)
        SF_min = np.min(SFMi)

        Thresh_E = Energy_PrimThresh*np.log10(E_min)
        Thresh_F = F_PrimThresh
        Thresh_SF = SF_PrimThresh

        count = 0

        if (Ei - E_min) >= Thresh_E:
            count += 1
        if (Fi - F_min) >= Thresh_F:
            count += 1
        if (SFMi - SF_min) >= Thresh_SF:
            count += 1

        if count > 1:
            speechCount += 1
            silenceCount = 0
        else:
            E_min = (silenceCount*E_min + Ei) / (silenceCount + 1)
            silenceCount += 1
            speechCount = 0
        Thresh_E = Energy_PrimThresh*np.log10(E_min)

        if speechCount >= 5:
            speech = True

        return speech


# Cálculo de la energía a corto plazo
def shortTermEnergy(frame):
    return sum([abs(x) ** 2 for x in frame]) / len(frame)


# Cálculo de la frecuencia dominante
def dominantFrequency(frame, CHUNK = 480, RATE = 48000):
    fft_wave = np.fft.fft(frame)
    idx = np.argmax(abs(fft_wave))
    freq = np.fft.fftfreq(CHUNK, 1)
    return abs(RATE*freq[int(idx)])


# Cálculo medida de planitud espectral
def SFM(frame, CHUNK=480):
    fft_wave = np.zeros(CHUNK)
    fft_wave = abs(np.fft.fft(frame))
    am = statistics.mean(fft_wave)
    gm = statistics.geometric_mean(fft_wave)
    return 10*np.log10(gm/am)


# Ejemplo main para detección voz

'''
# Constantes e inicialización pyaudio
CHUNK = 480  # Tamaño en muestras almacenadas en cada array. 10 ms para detección voz.
RATE = 48000  # Muestras por segundo
p = pyaudio.PyAudio()
stream = p.open(format=pyaudio.paFloat32, rate=RATE, channels=1, input=True, input_device_index=2,
                frames_per_buffer=CHUNK)  # Input Device = 0 si ordenador, 1 si cascos, 2 si Raspberry


def captarAudio(corr_factor = 4.73):
    # Almacenar audio en array
    data = np.frombuffer(stream.read(CHUNK, exception_on_overflow=False), dtype=np.float32)
    # Factor de corrección del dispositivo que está siendo utilizado para captar la señal. 94 dB a 1 kHz 1 Pa de referencia.
    data = data * corr_factor
    return data

try:
    # Bucle para captar audio de forma continua
    N = 0
    frames = []
    while True:  
        # Captación audio
        data = captarAudio()
        frames.append(data)
        N += 1

        if N == 50:
           	speech = VAD(frames)
        	if speech:
        		print('Voz detectada')
        	else:
        		print('Silencio')
        	N=0
        	frames=[]

# Salir del bucle con ctr+c
except KeyboardInterrupt:
    # Cierre stream
    stream.stop_stream()
    stream.close()
    p.terminate
    print('Interrupción de teclado. Finalizando programa.')
    sys.exit()
'''


# -*- coding: utf-8 -*-
import sys

sys.path.append('../')
"""
@author: Gabriel Mittag, TU-Berlin
"""
from MOS_INFER.NISQA_model import nisqaModel
import os
import warnings

os.environ['CUDA_VISIBLE_DEVICES'] = '0'
warnings.filterwarnings('ignore')

def cal_mos_infer(testFile=None):
    """
    Parameters
    ----------
    testFile

    Returns
    -------

    """
    args = {'mode': 'predict_file', 'pretrained_model': 'SC_res_retrain_220316_185754125621__ep_007.tar',
     'deg': '', 'data_dir': 'test', 'output_dir': None, 'csv_file': 'test-Copy1.csv',
     'csv_deg': 'filename', 'num_workers': 0, 'bs': 1, 'ms_channel': None, 'tr_bs_val': 1, 'tr_num_workers': 0,
     'task_type': 1}
    args['deg'] = testFile
    nisqa = nisqaModel(args)
    df = nisqa.predict()
    try:
        npyfile = testFile[:-4] + '_mel48.npy'
        os.remove(npyfile)
    except:
        print('AI MOS file delete err!')
    return df['mos_pred'].values[0]


if __name__ == "__main__":
    test = 'mixDstFile_minus_6.wav'
    df = cal_mos_infer(test)
    print(df)





































import ctypes
import sys
from ctypes import *


class pesqstruct(Structure):
 _fields_ = [
          ("pesq", c_double),  # c_byte
 ]

def cal_pesq(reffile, testfile,samplerate):
    pesqs = pesqstruct()

    import platform
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/PY_PESQ.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/PY_PESQ.dylib')

    inputFile = c_char_p(bytes(reffile.encode('utf-8')))#create_unicode_buffer(inFile.encode('utf-8'), len(inFile))
    outputFile = c_char_p(bytes(testfile.encode('utf-8')))#create_unicode_buffer(outFile.encode('utf-8'), len(outFile))
    if samplerate == 8000:
        mode = 'nb'
    if samplerate == 16000:
        mode = 'wb'
    cmode  = c_char_p(bytes(mode.encode('utf-8')))
    mydll.cal_pesq(inputFile,outputFile,samplerate,cmode,byref(pesqs))
    return pesqs.pesq


if __name__ == '__main__':
    ref = r'E:\files\out16000_8000.pcm'
    test = r'E:\files\out16000_8000.pcm'


    print(cal_pesq(ref,test,8000))


# -*- coding: UTF-8 -*-
import  sys
import os
from os import path
sys.path.append((os.path.dirname(path.dirname(__file__))))
from ctypes import *
import sys
import  time
from commFunction import global_result,getip

params = [1, 16000, 16, 99, 1, 16000, 16, 99, 1, 1, 0.2, 1, 0, 1, 0, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
          -1, -1]  # for WB 16K sample rate
params = [1, 16000, 16, 99, 1, 16000, 16, 99, 1, 1, 0.2, 1, 0, 1, 0, -1, -1, -1, -1, -1, -1, 0, 2, 0, 3, 0, 0, 0, 0, 0]

connect_flag = False
stop_flag = False
stop_flag1 = False
VQTdll = cdll.LoadLibrary(sys.prefix + '/VQTDll.dll')


def VQTLogfunc(msg):
    # print('VQTLogfunc : Start')
    c_s = msg.decode()
    # cs = str(c_s,'utf-8')
    cs = str(c_s)
    print(cs)
    global connect_flag
    global stop_flag1
    if 'Connected' in cs or 'Message sent' in cs:
        connect_flag = True
        stop_flag1 = True


def VQTMSGfunc(msg):
    c_s = string_at(msg)
    cs = str(c_s, 'utf-8')
    print(cs)
    global stop_flag
    if 'VQT PAMS/PSQM Test Failed' in cs:
        mosFailedFlag = True
    else:
        if 'POLQA:' in cs:
            global_result.mosResult['mos'] = cs.split('POLQA:')[1].strip()
        if 'Speech Level Gain:' in cs:
            global_result.mosResult['Speech Level Gain'] = cs.split('Speech Level Gain:')[1].strip()
        if 'Noise Level Gain:' in cs:
            global_result.mosResult['Noise Level Gain'] = cs.split('Noise Level Gain:')[1].strip()


def startvqt(srcFile=None, degFile=None, samplerate=48000):
    params = [1, 0, 16, 99, 1, 0, 16, 99, 1, 1, 0.2, 1, 0, 1, 0, 2]
    params = [1, 16000, 16, 99, 1, 16000, 16, 99, 1, 1, 0.2, 1, 0, 1, 0, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
              -1, -1, -1]
    params = [1, 16000, 16, 99, 1, 16000, 16, 99, 1, 1, 0.2, 1, 0, 1, 0, -1, -1, -1, -1, -1, -1, 0, 2, 0, 3, 0, 0, 0, 0,
              0]
    params[1], params[5] = samplerate, samplerate
    params = (c_float * len(params))(*params)
    if connect_flag == False:
        vip = getip()
        try:
            VQTdll.ConnectPort(c_char_p(vip.encode('utf_8')), 6666)
            while connect_flag == False:
                pass
        except Exception as e:
            print(str(e))
    VQTdll.RunVQTPAMSPSQM(0, 0, c_char_p(srcFile.encode('utf_8')), c_char_p(degFile.encode('utf_8')), 1, params)
    time.sleep(5)
    pass


def vqtDisConnect():
    VQTdll.Disconnect()


callback_type = CFUNCTYPE(None, c_char_p)
callback_login = callback_type(VQTLogfunc)
VQTdll.SetVQTLogMessage(callback_login)
# TODO 初始化放到init里
callback_type1 = CFUNCTYPE(None, c_wchar_p)
callback = callback_type1(VQTMSGfunc)
VQTdll.SetVQTRecvDBResponse(callback)


def test_polqa(file_name, count):
    f = file_name
    mode = ['_music', '_speech']
    for m in mode:
        bps = ['20kbps', '28kbps', '32kbps', '40kbps', '48kbps', '64kbps']
        for b in bps:
            for i in range(count):
                input = 'E:\\weiwei\\data_for_opus\\input\\' + f + str(i) + '.wav'
                output = 'E:\\weiwei\\data_for_opus\\output\\' + f + '_comp5_' + b + m + str(i) + '.wav'
                # print(input, output)
                startvqt(srcFile=input, degFile=output, samplerate=48000)
                print(output, mosResult['mos'])


if __name__ == '__main__':
    input = r'E:\01_MOS_AUTO_TESTER\testSrcFiles\Speech_16000\female\femalePolqaWB.pcm'
    output = r'E:\02_POLQA_RESULT\testDstFiles\NERTC_honor8x_iphone11_V4.3.0\L\Speech_16000\NONE\female\femalePolqaWB\femalePolqaWB_20210601201611_O_ManualTest_My Phone2_20210601201611_p_0.0.pcm'
    output2 = r'E:\02_POLQA_RESULT\testDstFiles\NERTC_honor8x_iphone11_V4.3.0\L\Speech_16000\NONE\female\femalePolqaWB\femalePolqaWB_20210601201541_O_ManualTest_My Phone2_20210601201541_p_4.19.pcm'
    output3 = r'E:\02_POLQA_RESULT\testDstFiles\123.pcm'
    startvqt(srcFile=input, degFile=output3,
             samplerate=16000)
    print(global_result.mosResult['mos'])
    VQTdll.Disconnect()



# -*- coding: UTF-8 -*-
import os
import time
import wave
import numpy as np

def wav2pcm(wavfile, data_type=np.int16):
    """
    :param wavfile:
    :param data_type:
    :return:
    """
    suffix = os.path.splitext(wavfile)[-1]
    if suffix == '.pcm':
        return wavfile
    if suffix != '.wav':
        raise TypeError('wrong format! not wav file!' + str(suffix))
    newFileName = wavfile[:-4] + '.pcm'
    f = open(wavfile, "rb")
    f.seek(0)
    f.read(44)
    data = np.fromfile(f, dtype= data_type)
    data.tofile(newFileName)
    f.close()
    return newFileName
    #os.remove(wavfile)


def pcm2wav(pcm_file, channels=1, bits=16, sample_rate=16000):
    """
    :param pcm_file:
    :param channels:
    :param bits:
    :param sample_rate:
    :return:
    """
    suffix = os.path.splitext(pcm_file)[-1]
    if suffix == '.wav':
        return pcm_file
    if suffix != '.pcm':
        raise TypeError('wrong format! not pcm file!' + str(suffix))
    newFileName = pcm_file[:-4] + '.wav'
    pcmf = open(pcm_file, 'rb')
    pcmdata = pcmf.read()
    pcmf.close()
    if bits % 8 != 0:
        raise ValueError("bits % 8 must == 0. now bits:" + str(bits))
    wavfile = wave.open(newFileName, 'wb')
    wavfile.setnchannels(channels)
    wavfile.setsampwidth(bits // 8)
    wavfile.setframerate(sample_rate)
    wavfile.writeframes(pcmdata)
    wavfile.close()
    time.sleep(1)
    #os.remove(pcm_file)
    return newFileName





if __name__ == '__main__':
    cle = r'E:\files\cle_malePolqaWB.wav'
    wav2pcm(cle)


# -*- coding: utf-8 -*-
# Andrea Castiella Aguirrezabala
# Roughness Zwicker y Fastl "Psychoacoustics: Facts and Models"

import numpy as np

# Cálculo Roughness
def acousticRoughness(specificLoudness, fmod=70):
    specificLoudnessdiff = np.zeros(len(specificLoudness))
    for i in range(len(specificLoudness)):
        if i == 0:
            specificLoudnessdiff[i] = specificLoudness[i]
        else:
            specificLoudnessdiff[i] = abs(specificLoudness[i] - specificLoudness[i-1])
    R = 0.3**(fmod/1000)*sum(0.1*specificLoudnessdiff)
    return R


import socket,json
import time

# class SocketClient:
#     def __init__(self, ip: str, port: int):
#         self.s = socket.socket()
#         self.s.connect((ip, port))
#
#     def sender(self, data: dict)->json:
#         data = json.dumps(data)
#         self.s.send(bytes(data.encode("utf-8")))
#         return json.loads(str(self.s.recv(1024), encoding="utf-8"))
#
#     def close(self):
#         self.s.close()



class SocketClient:
    def __init__(self, ip: str, port: int):
        time.sleep(2)
        self.s = socket.socket()
        self.s.connect((ip, port))
        self.recv_data = b''

    def sender(self, data: dict) -> json:
        data = json.dumps(data)
        self.s.send(bytes(data.encode("utf-8")))
        #return str(self.s.recv(1024), encoding="utf-8")
        #print(self.s.recv(1024).decode('utf-8'))
        # print(str(self.s.recv(1024), encoding="utf-8"))
        #print(self.s.recv(1024))
        #return json.loads(self.s.recv(1024))
        return json.loads(str(self.s.recv(1024), encoding="utf-8"))


    def close(self):
        self.s.close()



if __name__ == '__main__':
    socket = SocketClient('10.242.167.159',2159)
    data = {"type": "command", "module": "client"}
    data["method"] = "JoinChannle"
    data["token"] = ""
    res = socket.sender(data)
    print(res)
    socket.close()


# -*- coding: UTF-8 -*-
import struct
import ctypes
import sys
import time
from ctypes import *


class p563Struct(Structure):
 _fields_ = [
          ("fSpeechLevel", c_double),  # c_byte
          ("fPitchAverage", c_double),  # c_byte
          ("fNoiseLevel", c_double),  #  c_byte
          ("fSnr", c_double),  # c_byte
          ("fRobotisation", c_double),  #  c_byte
          ("fPredictedMos", c_double)  # c_byte
     ]


def cal_563_mos(degFile):
    """
    仅支持窄带信号 （8k、16bit、mono）
    :param degFile: 输入文件
    :return: 无参考的MOS,语音电平，信噪比，噪声电平
    """
    p563 = p563Struct()

    import platform
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/p563.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/p563.dylib')

    #buf = create_string_buffer(degFile.encode('gbk'), len(degFile))
    buf = c_char_p(bytes(degFile.encode('utf-8')))
    mydll.NR_MOS(buf,byref(p563))
    return p563.fPredictedMos,p563.fSpeechLevel,p563.fSnr,p563.fNoiseLevel

if __name__ == '__main__':
    for a in range(100):
        time.sleep(20)
        src = r'E:\audioalgorithm\audiotestalgorithm\demos\02_p563_demo\cleDstFile.wav'
        print(cal_563_mos(src))


# -*- coding:utf-8 -*-
import copy
import sys
import  os
from os import path
sys.path.append(os.path.dirname(path.dirname(__file__)))
sys.path.append(path.dirname(__file__))
from commFunction import getip,log_time,global_result,sftp_connect,sftp_put,sftp_disconnect,get_file_path
import shutil
from  socketClient import SocketClient
import datetime



def polqa_client_test(src,test,samplerate):
    curip = getip()
    curtime = log_time()
    curpath = str(curip) + '_'+str(curtime)
    os.mkdir(curpath)

    curdata = global_result.get_data()
    curdata['module'] = 'clientA'
    curdata['method'] = 'requestA'
    curdata['samplerate'] = samplerate
    curdata['token'] = curpath
    curdata['srcFile'] = os.path.basename(src)
    curdata['testFile'] = os.path.basename(test)

    #ssh
    shutil.copy(src,curpath)
    shutil.copy(test, curpath)

    dstpath = '/home/netease/polqa'


    # stfp
    client,sftp = sftp_connect(global_result.username,global_result.password,global_result.HOST,port=global_result.sftpPort)
    sftp_put(sftp,curpath, dstpath)
    sftp_disconnect(client)

    shutil.rmtree(curpath)
    # get result
    socket = SocketClient(global_result.machost,global_result.PORT)
    try:
        result = socket.sender(curdata)
        print(result['result'])
    except:
        socket.close()
        return None
    return  result['result']


if __name__ == '__main__':
    import platform
    print(platform.uname())
    print(sys.platform)
    exit(0)
    # client, sftp = sftp_connect(username, password, serverIP, port=port)
    # sftp_get(sftp, '/home/netease/polqa_result/' + '192.168.1.3_2021-08-18-17-52-35' + '.ress', 'result')
    src = r'D:\0\speech_cn_minus_13.pcm'
    test1 = r'D:\0\mixDstFile_minus_13.pcm'
    test2 = r'D:\0\mixDstFile_minus_13_-6.pcm'
    refout = r'E:\AIVAD\OTHER\ref_out0_01.pcm'
    file = r'E:\AIVAD\OTHER\reverse0_01.pcm'
    info = polqa_client_test(src, test1, 48000)
    info = polqa_client_test(src, test2, 48000)
    exit(0)
    path = r'E:\02_POLQA_RESULT\testDstFiles\NERTC_iphone11_honor8x_V4.3.0\L\Speech_48000\NONE\female\femalePOLQASWB'
    src = r'E:\01_MOS_AUTO_TESTER\testSrcFiles\Speech_48000\female\femalePOLQASWB.pcm'
    filelist = []
    f = open('time.txt','w')

    get_file_path(path,filelist,[])
    print(filelist)
    #print(exists_remote('10.219.33.45','/home/netease/polqa_result/455.ss'))
    for a in range (1000):
        for file in filelist:
            print('*****************************************************')
            print('curent testfile name is {0},srcfile name is {1}'.format(file,src))
            begin_time = datetime.datetime.now()
            info = polqa_client_test(src, file,48000)
            end_time = datetime.datetime.now()
            duration = end_time - begin_time
            print('time duration is {}'.format(duration))
            f.writelines('*****************************************************\n')
            f.writelines('file name is {}\n'.format(file))
            f.writelines('result is {}\n'.format(str(info)))
            f.writelines('time duration is {}\n'.format(duration))
    f.close()
    #vqtDisConnect
    # for a in range(100):
    #     polqa_client_test(srcfile,testfile)



# -*- coding: UTF-8 -*-
import sys

sys.path.append('../')
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_
import ctypes


# void SNR_transient(const emxArray_real_T *ref, const emxArray_real_T *ref_noise,
#                    const emxArray_real_T *sig, double fs, double *SNR, double
#                    *noise_dB, double *err)

def cal_snr_transient(refFile=None, noisetFile=None, testFile=None):
    """
    """
    refstruct,refsamplerate,_ = get_data_of_ctypes_(refFile,True)
    teststruct,testsamplerate,_ = get_data_of_ctypes_(testFile,True)
    noiseStruct,noisesamplerate,_ = get_data_of_ctypes_(noisetFile,True)
    if refsamplerate != testsamplerate or refsamplerate!= noisesamplerate:
        raise TypeError('Different format of ref and test files!')

    import platform
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/snr_transient.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/snr_transient.dylib')

    mydll.SNR_transient.argtypes = [POINTER(emxArray_real_T),POINTER(emxArray_real_T),POINTER(emxArray_real_T),c_double, POINTER(c_double),POINTER(c_double),POINTER(c_double)]
    snr_1,snr_2,err = c_double(0.0),c_double(0.0),c_double(0.0)
    mydll.SNR_transient(byref(refstruct),byref(noiseStruct),byref(teststruct),c_double(refsamplerate),byref(snr_1),byref(snr_2),byref(err))

    if err.value == 0.0:
        return snr_1.value,snr_2.value
    else:
        return None


if __name__ == '__main__':
    print(sys.prefix)
    speech = r'C:\Users\vcloud_avl\Documents\我的POPO\nearend_ref.wav'
    noise = r'C:\Users\vcloud_avl\Documents\我的POPO\noise.wav'
    test = r'C:\Users\vcloud_avl\Documents\我的POPO\3_noise_suppression_1655194712.wav'
    print(cal_snr_transient(refFile=speech,noisetFile=noise,testFile=test))

    pass



import ctypes
import sys
import  os
from os import path
sys.path.append(os.path.dirname(path.dirname(__file__)))
import wave
import  time
from ctypes import *
from commFunction import get_data_array
import numpy as np

def resample(infile,target_amplerate,outfile=None):
    data,fs,_ = get_data_array(infile)
    print(data)
    if fs == target_amplerate:
        return infile
    #uint64_t Resample_s16(const int16_t *input, int16_t *output, int inSampleRate, int outSampleRate, uint64_t inputSize,uint32_t channels)

    import platform
    #mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/resampler.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/resample.dylib')
    if outfile is None:
        outfile = infile[:-4] +'_' +str(target_amplerate) + '.wav'
    infile_ref = c_char_p(bytes(infile.encode('utf-8')))
    outfile_ref = c_char_p(bytes(outfile.encode('utf-8')))
    mydll.resample2file(infile_ref,outfile_ref,target_amplerate)
    return outfile
    pass
# 多声道转1声道 采样率转换
def restruct(infile,target_amplerate,outfile=None):
    onechannelfile = infile
    data,fs,chn = get_data_array(infile)
    if fs == target_amplerate and chn == 1:
        return infile
    if chn != 1:
        onechannelfile = infile[:-4] + '_mono.wav'
        data =  np.array([data[n] for n in range(len(data)) if n%chn==0])
        wavfile = wave.open(onechannelfile, 'wb')
        wavfile.setnchannels(1)
        wavfile.setsampwidth(2)
        wavfile.setframerate(fs)
        wavfile.writeframes(data.tobytes())
        wavfile.close()
        time.sleep(1)
    #uint64_t Resample_s16(const int16_t *input, int16_t *output, int inSampleRate, int outSampleRate, uint64_t inputSize,uint32_t channels)

    return resample(onechannelfile,target_amplerate,outfile=outfile)
    pass

if __name__ == '__main__':
    dst = r'E:/02_ai_vad/aivad-seqs/Speech/T0055G0092S0001.wav'
    noise = r'E:/02_ai_vad/aivad-seqs/Noise/car_horn-1.wav'
    sam = 8000
    print(restruct(noise,sam,'1.wav'))



# -*- coding: UTF-8 -*-
import sys
sys.path.append('../')
import ctypes
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_

# void SNR_estimation(const emxArray_real_T *ref, const emxArray_real_T *sig,
#                     double fs, double *SNR, double *err)


def cal_snr_music(refFile=None, testFile=None):
    """
    """
    refstruct,refsamplerate,_ = get_data_of_ctypes_(refFile,True)
    teststruct,testsamplerate,_ = get_data_of_ctypes_(testFile,True)
    if refsamplerate != testsamplerate:
        raise TypeError('Different format of ref and test files!')

    import platform
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/snr_music.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/snr_music.dylib')

    mydll.SNR_estimation.argtypes = [POINTER(emxArray_real_T),POINTER(emxArray_real_T),c_double, POINTER(c_double),POINTER(c_double)]
    snr,err = c_double(0.0),c_double(0.0)
    mydll.SNR_estimation(byref(refstruct),byref(teststruct),c_double(refsamplerate),byref(snr),byref(err))

    if err.value == 0.0:
        return snr.value
    else:
        return None

if __name__ == '__main__':
    cal_snr_music(refFile='speech.wav',testFile='music_rap.wav')
    pass


import wave
import os
import  numpy as np
from moviepy.editor import AudioFileClip
from commFunction import get_rms,get_ave_rms,get_one_channel_data,get_file_duration,get_data_array
from formatConvert import pcm2wav

def get_wav_from_mp4(mp4file):
    """
    Parameters
    ----------
    mp4file

    Returns
    -------

    """
    suffix = os.path.splitext(mp4file)[-1]
    if suffix != '.mp4':
        raise TypeError('wrong format! not mp4 file!' + str(suffix))
    my_audio_clip = AudioFileClip(mp4file)
    newFileName = mp4file[:-4] + '.wav'
    my_audio_clip.write_audiofile(newFileName)
    return newFileName


def isSlience(Filename =None,section=None,channels=1, bits=16, sample_rate=16000):
    """
    Parameters
    ----------
    Filename 支持 wav 和 pcm 和MP4

    Returns
    -------

    """
    suffix = os.path.splitext(Filename)[-1]

    if suffix == '.mp4':
        Filename = get_wav_from_mp4(Filename)
    if suffix == '.pcm':
        Filename = pcm2wav(Filename,channels,bits,sample_rate)
    if suffix == '.wav':
        pass
    lenth,fs = get_file_duration(Filename)
    data = get_one_channel_data(Filename)
    if section is None:
        startTime = 0
        endTime = lenth
    else:
        startTime = section[0]
        endTime = section[1]
    if startTime > lenth or startTime > endTime:
        raise TypeError('start point is larger than the file lenth :' + str(suffix))
    if endTime > lenth:
        endTime = lenth
    ins = data[int(startTime*fs):int(endTime*fs)]

    dBrmsValue = get_rms(ins)#20*math.log10(get_rms(ins)/32767+ 1.0E-6)
    print(dBrmsValue)
    if dBrmsValue > -70:
        return False
    else:
        for n in range(len(ins)//480):
            curdata = ins[480*n:480*(n+1)]
            dBrmsValue = get_rms(curdata)#20 * math.log10(get_rms(curdata) / 32767 + 1.0E-6)
            print(dBrmsValue)
            if dBrmsValue > -60:
                return False
        return True
    pass


def audioFormat(wavFileName=None):
    """
    wavFileName：输入文件 wav，mp4
    Returns
    -------
    refChannel:通道数
    refsamWidth：比特位 2代表16bit
    refsamplerate：采样率
    refframeCount：样点数
    """
    suffix = os.path.splitext(wavFileName)[-1]
    if suffix != '.wav' and suffix != '.mp4':
        raise TypeError('wrong format! not wav/mp4 file!' + str(suffix))
    if suffix == '.mp4':
        wavFileName = get_wav_from_mp4(wavFileName)
    wavf = wave.open(wavFileName, 'rb')
    refChannel,refsamWidth,refsamplerate,refframeCount = wavf.getnchannels(),wavf.getsampwidth(),wavf.getframerate(),wavf.getnframes()
    return refChannel,refsamWidth*8,refsamplerate,refframeCount

def get_rms_level(wavFileName=None,rmsMode='total',section=None):
    """
    wavFileName：输入文件 wav，mp4
    Returns
    -------
    refChannel:通道数
    refsamWidth：比特位 2代表16bit
    refsamplerate：采样率
    refframeCount：样点数
    """
    suffix = os.path.splitext(wavFileName)[-1]
    if suffix != '.wav':
        raise TypeError('wrong format! not wav file!' + str(suffix))

    lenth,fs = get_file_duration(wavFileName)
    data = get_one_channel_data(wavFileName)
    if section == None:
        startTime = 0
        endTime = lenth
    else:
        startTime = section[0]
        endTime = section[1]
    if startTime > lenth or startTime > endTime:
        raise TypeError('start point is larger than the file lenth :' + str(suffix))
    if endTime > lenth:
        endTime = lenth
    curdata = data[int(startTime*fs):int(endTime*fs)]
    if rmsMode == 'total':
        return get_rms(curdata)
    if rmsMode == 'average':
        return get_ave_rms(curdata)
    return None



if __name__ == '__main__':
    ref = r'E:\02_ai_vad\5_aecm_1648612346.wav'
    print(isSlience(ref,section=[0,20]),)
    #print(audioFormat(ref))


# -*- coding: UTF-8 -*-
import sys
sys.path.append('../')
import ctypes
from ctypes import *




class g160Struct(Structure):
 _fields_ = [
          ("tnlr", c_double),  # c_byte
          ("nplr", c_double),  # c_byte
          ("snri", c_double),  #  c_byte
          ("dsn", c_double),  # c_byte
     ]


def cal_g160(cleanFile,inFile,outFile,inOffset,outOffset,maxComNLevel=-48.0,speechPauseLevel=-35.0):
    """
    :param cleanFile: 干净语音文件
    :param inFile:  输入带噪语音文件
    :param outFile:  输出文件
    :param inOffset:  输入文件的样点延迟
    :param outOffset:   输出文件的样点延迟
    :param maxComNLevel:  最大舒适噪声，默认-48dbov
    :param speechPauseLevel:  非语音段最大的电平门限 -35dbov
    :return:
    """
    g160 = g160Struct()

    import platform
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/g160.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/g160.dylib')

    cleFile = c_char_p(bytes(cleanFile.encode('utf-8')))#create_unicode_buffer(cleanFile.encode('utf-8'), len(cleanFile))
    inputFile = c_char_p(bytes(inFile.encode('utf-8')))#create_unicode_buffer(inFile.encode('utf-8'), len(inFile))
    outputFile = c_char_p(bytes(outFile.encode('utf-8')))#create_unicode_buffer(outFile.encode('utf-8'), len(outFile))
    mydll.Noise_Compute(cleFile,inputFile,outputFile,inOffset,outOffset,c_double(maxComNLevel),c_double(speechPauseLevel),byref(g160))
    return g160.tnlr,g160.nplr,g160.snri,g160.dsn

if __name__ == '__main__':
    print(cal_g160(r'E:\files\cle_malePolqaWB.wav',r'E:\files\malePolqaWB.wav',r'E:\files\test_malePolqaWB.wav',192,192))


# -*- coding: UTF-8 -*-
import sys

sys.path.append('../')
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_
import ctypes
import platform

# void SNR_transient(const emxArray_real_T *ref, const emxArray_real_T *ref_noise,
#                    const emxArray_real_T *sig, double fs, double *SNR, double
#                    *noise_dB, double *err)


# void attackrelease_estimation(const emxArray_real_T *ref, const emxArray_real_T *
#   sig, double fs_ref, double fs_sig, double *time_attack, double *time_release,
#   double *err)
def cal_attack_release(refFile=None, testFile=None):
    """
    """
    refstruct,refsamplerate,_ = get_data_of_ctypes_(refFile,True)
    teststruct,testsamplerate,_ = get_data_of_ctypes_(testFile,True)

    if refsamplerate != testsamplerate :
        raise TypeError('Different format of ref and test files!')

    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/attackrelease.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/attackrelease.dylib')

    mydll.attackrelease_estimation.argtypes = [POINTER(emxArray_real_T),POINTER(emxArray_real_T),c_double,c_double, POINTER(c_double),POINTER(c_double),POINTER(c_double)]
    time_attack,time_release,err = c_double(0.0),c_double(0.0),c_double(0.0)
    mydll.attackrelease_estimation(byref(refstruct),byref(teststruct),c_double(refsamplerate),c_double(refsamplerate),byref(time_attack),byref(time_release),byref(err))

    if err.value == 0.0:
        return time_attack.value,time_release.value
    else:
        return None


if __name__ == '__main__':
    file = r'C:\Users\vcloud_avl\Documents\我的POPO\0\speech_attackrelease.wav'
    test = r'C:\Users\vcloud_avl\Documents\我的POPO\0\speech_attackrelease.wav'
    print(cal_attack_release(refFile=file,testFile=test))
    pass


#!/usr/bin/python

"""
Speech Transmission Index (STI) from speech waveforms (real speech)

Copyright (C) 2011 Jon Polom <jmpolom@wayne.edu>
Licensed under the GNU General Public License
"""

from datetime import date, datetime
from matplotlib.mlab import cohere,psd
from numpy import append,array,clip,log10,nonzero,ones,power,reshape
from numpy import searchsorted,shape,sqrt,sum,vstack,zeros
from numpy.ma import masked_array
from scipy.io import wavfile
from scipy.signal import butter,firwin,decimate,lfilter
from sys import stdout
from warnings import catch_warnings,simplefilter

__author__ = "Jonathan Polom <jmpolom@wayne.edu>"
__date__ = date(2011, 4, 22)
__version__ = "0.5"

def thirdOctaves(minFreq, maxFreq):
    """
    Calculates a list of frequencies spaced 1/3 octave apart in hertz
    between minFreq and maxFreq
    
    Input
    -----
    * minFreq : float or int
    
        Must be non-zero and non-negative

    * maxFreq : float or int
    
        Must be non-zero and non-negative
    
    Output
    ------
    * freqs : ndarray
    """

    if minFreq <= 0 or maxFreq <= 0:
        raise ValueError("minFreq and maxFreq must be non-zero and non-negative")
    else:
        maxFreq = float(maxFreq)

        f = float(minFreq)
        freqs = array([f])

        while f < maxFreq:
            f = f * 10**0.1
            freqs = append(freqs, f)

        return freqs

def fftWindowSize(freqRes, hz):
    """
    Calculate power of 2 window length for FFT to achieve specified frequency
    resolution. Useful for power spectra and coherence calculations.
    
    Input
    -----
    * freqRes : float
    
        Desired frequency resolution in hertz
    
    * hz : int
    
        Sample rate, in hertz, of signal undergoing FFT
    
    Output
    ------
    * window : int
    """
    
    freqRes = float(freqRes)         # make sure frequency res is a float
    pwr = 1                          # initial power of 2 to try
    res = hz / float(2**pwr) # calculate frequency resolution
    
    while res > freqRes:
        pwr += 1
        res = hz / float(2**pwr)
    
    return 2**pwr

def downsampleBands(audio, hz, downsampleFactor):
    """
    Downsample audio by integer factor
    
    Input
    -----
    * audio : array-like
    
        Array of original audio samples
    
    * hz : float or int
    
        Original audio sample rate in hertz
    
    * downsampleFactor : int
    
        Factor to downsample audio by, if desired
    
    Output
    ------
    * dsAudio : ndarray
    
        Downsampled audio array
    
    * hz : int
    
        Downsampled audio sample rate in hertz
    """

    # calculate downsampled audio rate in hertz
    downsampleFactor = int(downsampleFactor)        # factor must be integer
    hz = int(hz / downsampleFactor)
    
    for band in audio:
        ds = decimate(band, downsampleFactor, ftype='fir')
        
        try:
            dsAudio = append(dsAudio, ds)
        except:
            dsAudio = ds
    
    return dsAudio, hz

def octaveBandFilter(audio, hz, 
                     octaveBands=[125, 250, 500, 1000, 2000, 4000, 8000],
                     butterOrd=6, hammingTime=16.6):

    """
    Octave band filter raw audio. The audio is filtered through butterworth
    filters of order 6 (by default), squared to obtain the envelope and finally
    low-pass filtered using a 'hammingTime' length Hamming filter at 25 Hz.
    
    Input
    -----
    * audio : array-like
    
        Array of raw audio samples
    
    * hz : float or int
    
        Audio sample rate in hertz
    
    * octaveBands : array-like
    
        list or array of octave band center frequencies
    
    * butterOrd : int
    
        butterworth filter order
    
    * hammingTime : float or int
    
        Hamming window length, in milliseconds relative to audio sample rate
    
    Output
    ------
    * octaveBandAudio : ndarray
    
        Octave band filtered audio
    
    * hz : float or int
    
        Filtered audio sample rate
    """

    print("Butterworth filter order:",butterOrd)
    print("Hamming filter length:   ",hammingTime,"milliseconds")
    print ("Audio sample rate:       ",hz)

    # calculate the nyquist frequency
    nyquist = hz * 0.5

    # length of Hamming window for FIR low-pass at 25 Hz
    hammingLength = (hammingTime / 1000.0) * hz
    print(hammingLength)
    # process each octave band
    for f in octaveBands:
        bands = str(octaveBands[:octaveBands.index(f) + 1]).strip('[]')
        statusStr = "Octave band filtering audio at: " + bands
        unitStr = "Hz ".rjust(80 - len(statusStr))
        stdout.write(statusStr)
        stdout.write(unitStr)
        stdout.write('\r')
        stdout.flush()
    
        # filter the output at the octave band f
        f1 = f / sqrt(2)
        f2 = f * sqrt(2)

        # for some odd reason the band-pass butterworth doesn't work right
        # when the filter order is high (above 3). likely a SciPy issue?
        # also, butter likes to complain about possibly useless results when
        # calculating filter coefficients for high order (above 4) low-pass
        # filters with relatively low knee frequencies (relative to nyquist F).
        # perhaps I just don't know how digital butterworth filters work and
        # their limitations but I think this is odd.
        # the issue described here will be sent to their mailing list
        if f < max(octaveBands):
            with catch_warnings():      # suppress the spurious warnings given
                simplefilter('ignore')  # under certain conditions
                b1,a1 = butter(butterOrd, f1/nyquist, btype='high')
                b2,a2 = butter(butterOrd, f2/nyquist, btype='low')
            
            filtOut = lfilter(b1, a1, audio)   # high-pass raw audio at f1
            filtOut = lfilter(b2, a2, filtOut) # low-pass after high-pass at f1
        else:
            with catch_warnings():
                simplefilter('ignore')
                b1,a1 = butter(butterOrd, f/nyquist, btype='high')
            filtOut = lfilter(b1, a1, audio)

        filtOut = array(filtOut)**2
        b = firwin(int(hammingLength), 25.0, nyq=nyquist)
        filtOut = lfilter(b, 1, filtOut)
        filtOut = filtOut * -1.0

        # stack-up octave band filtered audio
        try:
            octaveBandAudio = vstack((octaveBandAudio, filtOut))
        except:
            octaveBandAudio = filtOut

    print()
    return octaveBandAudio

def octaveBandSpectra(filteredAudioBands, hz, fftRes=0.06):
    """
    Calculate octave band power spectras
    
    Input
    -----
    * filteredAudioBands : array-like
    
        Octave band filtered audio
    
    * hz : float or int
    
        Audio sample rate in hertz. Must be the same for clean and dirty audio
    
    * fftRes : float or int
    
        Desired FFT frequency resolution
    
    Output
    ------
    * spectras : ndarray
    
        Power spectra values
    
    * fftfreqs : ndarray
    
        Frequencies for FFT points
    """

    # FFT window size for PSD calculation: 32768 for ~0.06 Hz res at 2 kHz
    psdWindow = fftWindowSize(fftRes, hz)
    
    print("Calculating octave band power spectras",)
    print("(FFT length:",psdWindow,"samples)")

    for band in filteredAudioBands:        
        spectra, freqs = psd(band, NFFT=psdWindow, Fs=hz)
        spectra = reshape(spectra, len(freqs))  # change to row vector
        spectra = spectra / max(spectra)        # scale to [0,1]
        
        # stack-up octave band spectras
        try:
            spectras = vstack((spectras, spectra))
            fftfreqs = vstack((fftfreqs, freqs))
        except:
            spectras = spectra
            fftfreqs = freqs
        
    return spectras, fftfreqs

def octaveBandCoherence(degrAudioBands, refAudioBands,
                        hz, fftRes=0.122):
    """
    Calculate coherence between clean and degraded octave band audio
    
    Input
    -----
    * degrAudioBands : array-like
    
        Degraded octave band audio
    
    * refAudioBands : array-like
    
        Reference (clean) octave band audio
    
    * hz : float or int
    
        Audio sample rate. Must be common between clean and dirty audio
    
    * fftRes : float or int
    
        Desired FFT frequency resolution
    
    Output
    ------
    * coherences : ndarray
    
        Coherence values
    
    * fftfreqs : ndarray
    
        Frequencies for FFT points
    """

    # FFT window size for PSD calculation: 32768 for ~0.06 Hz res at 2 kHz
    # Beware that 'cohere' isn't as forgiving as 'psd' with FFT lengths 
    # larger than half the length of the signal
    psdWindow = fftWindowSize(fftRes, hz)
    
    print("Calculating degraded and reference audio coherence",)
    print ("(FFT length:",psdWindow,"samples)")

    for i,band in enumerate(degrAudioBands):
        with catch_warnings():      # catch and ignore spurious warnings
            simplefilter('ignore')  # due to some irrelevant divide by 0's
            coherence, freqs = cohere(band, refAudioBands[i], 
                                      NFFT=psdWindow, Fs=hz)
        
        # stack-up octave band spectras
        try:
            coherences = vstack((coherences, coherence))
            fftfreqs = vstack((fftfreqs, freqs))
        except:
            coherences = coherence
            fftfreqs = freqs
        
    return coherences, fftfreqs

def thirdOctaveRootSum(spectras, fftfreqs, minFreq=0.25, maxFreq=25.0):
    """
    Calculates square root of sum of spectra over 1/3 octave bands
    
    Input
    -----
    * spectras : array-like
    
        Array or list of octave band spectras
    
    * fftfreqs : array-like
    
        Array or list of octave band FFT frequencies
    
    * minFreq : float
    
        Min frequency in 1/3 octave bands
    
    * maxFreq : float
    
        Max frequency in 1/3 octave bands
    
    Output
    ------
    * thirdOctaveRootSums : ndarray
    
        Square root of spectra sums over 1/3 octave intervals
    """

    print ("Calculating 1/3 octave square-rooted sums from",)
    print (minFreq,"to",maxFreq,"Hz")

    thirdOctaveBands = thirdOctaves(minFreq, maxFreq)
        
    # loop over the spectras contained in 'spectras' and calculate 1/3 oct MTF
    for i,spectra in enumerate(spectras):
        freqs = fftfreqs[i]                # get fft frequencies for spectra

        # calculate the third octave sums
        for f13 in thirdOctaveBands:
            f131 = f13 / power(2, 1.0/6.0) # band start
            f132 = f13 * power(2, 1.0/6.0) # band end
            
            li = searchsorted(freqs, f131)
            ui = searchsorted(freqs, f132) + 1
            
            s = sum(spectra[li:ui]) # sum the spectral components in band
            s = sqrt(s)             # take square root of summed components
            
            try:
                sums = append(sums, s)
            except:
                sums = array([s])

        # stack-up third octave modulation transfer functions
        try:
            thirdOctaveSums = vstack((thirdOctaveSums, sums))
        except:
            thirdOctaveSums = sums
        
        # remove temp 'sum' and 'counts' variables for next octave band
        del(sums)

    return thirdOctaveSums

def thirdOctaveRMS(spectras, fftfreqs, minFreq=0.25, maxFreq=25.0):
    """
    Calculates RMS value of spectra over 1/3 octave bands
    
    Input
    -----
    * spectras : array-like
    
        Array or list of octave band spectras
    
    * fftfreqs : array-like
    
        Array or list of octave band FFT frequencies
    
    * minFreq : float
    
        Min frequency in 1/3 octave bands
    
    * maxFreq : float
    
        Max frequency in 1/3 octave bands
    
    Output
    ------
    * thirdOctaveRMSValues : ndarray
    
        RMS value of spectra over 1/3 octave intervals
    """

    print ("Calculating 1/3 octave RMS values from",)
    print (minFreq,"to",maxFreq,"Hz")

    thirdOctaveBands = thirdOctaves(minFreq, maxFreq)
        
    # loop over the spectras contained in 'spectras' and calculate 1/3 oct MTF
    for i,spectra in enumerate(spectras):
        freqs = fftfreqs[i]                # get fft frequencies for spectra

        # calculate the third octave sums
        for f13 in thirdOctaveBands:
            f131 = f13 / power(2, 1.0/6.0) # band start
            f132 = f13 * power(2, 1.0/6.0) # band end
            
            li = searchsorted(freqs, f131)
            ui = searchsorted(freqs, f132) + 1
            
            s = sum(spectra[li:ui]**2)  # sum the spectral components in band
            s = s / len(spectra[li:ui]) # divide by length of sum
            s = sqrt(s)                 # square root
            
            try:
                sums = append(sums, s)
            except:
                sums = array([s])

        # stack-up third octave modulation transfer functions
        try:
            thirdOctaveRMSValues = vstack((thirdOctaveRMSValues, sums))
        except:
            thirdOctaveRMSValues = sums
        
        # remove temp 'sum' and 'counts' variables for next octave band
        del(sums)

    return thirdOctaveRMSValues

def sti(modulations, coherences, minCoherence=0.8):
    """
    Calculate the speech transmission index from third octave modulation
    indices. The indices are truncated after coherence between clean and dirty
    audio falls below 'minCoherence' or 0.8, by default.
    
    Input
    -----
    * modulations : array-like
    
        Modulation indices spaced at 1/3 octaves within each octave band
    
    * coherences : array-like
    
        Coherence between clean and dirty octave band filtered audio
    
    * minCoherence : float
    
        The minimum coherence to include a mod index in the STI computation
    
    Output
    ------
    * index : float
    
        The speech transmission index (STI)
    """
    
    # create masking array of zeroes
    snrMask = zeros(modulations.shape, dtype=int)
    
    # sort through coherence array and mask corresponding SNRs where coherence
    # values fall below 'minCoherence' (0.8 in most cases and by default)
    for i,band in enumerate(coherences):
        lessThanMin = nonzero(band < minCoherence)[0]
        if len(lessThanMin) >= 1:
            discardAfter = min(lessThanMin)
            snrMask[i][discardAfter:] = ones((len(snrMask[i][discardAfter:])))
    
    modulations = clip(modulations, 0, 0.99)      # clip to [0, 0.99] (max: ~1)
    snr = 10*log10(modulations/(1 - modulations)) # estimate SNR
    snr = clip(snr, -15, 15)                      # clip to [-15,15]
    snr = masked_array(snr, mask=snrMask)         # exclude values from sum
    snrCounts = (snr / snr).sum(axis=1)           # count SNRs
    snrCounts = snrCounts.data                    # remove masking
    octaveBandSNR = snr.sum(axis=1) / snrCounts   # calc average SNR
    alpha = 7 * (snrCounts / snrCounts.sum())     # calc alpha weight

    # octave band weighting factors, Steeneken and Houtgast (1985)
    w = [0.129, 0.143, 0.114, 0.114, 0.186, 0.171, 0.143]
    
    # calculate the STI measure
    snrp = alpha * w * octaveBandSNR
    snrp = snrp.sum()
    index = (snrp + 15) / 30.0
    
    print ("Speech Transmission Index (STI):",index)
    return index

def stiFromAudio(reference, degraded, hz, calcref=False, downsample=None,
                 name="unnamed"):
    """
    Calculate the speech transmission index (STI) from clean and dirty
    (ie: distorted) audio samples. The clean and dirty audio samples must have
    a common sample rate for successful use of this function.
    
    Input
    -----
    * reference : array-like
    
        Clean reference audio sample as an array of floating-point values
    
    * degraded : array-like
    
        Degraded audio sample as an array, or array of arrays for multiple
        samples, of floating-point values
    
    * hz : int
    
        Audio sample rate in hertz
    
    * calcref : boolean
    
        Calculate STI for reference signal alone
    
    * downsample : int or None
    
        Downsampling integer factor
    
    * name : string
    
        Name of sample set, for output tracking in larger runs
    
    Output
    ------
    * sti : array-like or float
    
        The calculated speech transmission index (STI) value(s)
    """
    
    # put single sample degraded array into another array so the loop works
    if type(degraded) is not type([]):
        degraded = [degraded]
    
    print ("-"*80)
    print ("Speech Transmission Index (STI) from speech waveforms".center(80))
    print ("-"*80)
    print ()
    print ("Sample set:             ",name)
    print ("Number of samples:      ",len(degraded))
    print ("Date/time:              ",datetime.now().isoformat())
    print ("Calculate reference STI:",)
    if calcref:
        print ("yes")
    else:
        print ("no")
    print (" Reference Speech ".center(80,'*'))
    
    refOctaveBands = octaveBandFilter(reference, hz)
    refRate = hz

    # downsampling, if desired
    if type(downsample) is type(1):
        refOctaveBands, refRate = downsampleBands(refOctaveBands, refRate,
                                                  downsample)
    
    # calculate STI for reference sample, if boolean set
    if calcref:
        # STI calc procedure
        spectras, sfreqs = octaveBandSpectra(refOctaveBands, refRate)
        coherences, cfreqs = octaveBandCoherence(refOctaveBands, refOctaveBands,
                                                 refRate)
        thirdOctaveMTF = thirdOctaveRootSum(spectras, sfreqs)
        thirdOctaveCoherences = thirdOctaveRMS(coherences, cfreqs)
        
        # add to interim array for MTFs and coherences
        try:
            thirdOctaveTemps.append([thirdOctaveMTF, thirdOctaveCoherences])
        except:
            thirdOctaveTemps = [[thirdOctaveMTF, thirdOctaveCoherences]]
    
    print()
    
    # loop over degraded audio samples and calculate STIs
    for j,sample in enumerate(degraded):
        print (" Degraded Speech: Sample {0} ".format(j + 1).center(80,'*'))
        degrOctaveBands = octaveBandFilter(sample, hz)
        degrRate = hz
        
        # downsampling, if desired
        if type(downsample) is type(1):
            degrOctaveBands, degrRate = downsampleBands(degrOctaveBands, 
                                                        degrRate, downsample)
        
        # STI calc procedure
        spectras, sfreqs = octaveBandSpectra(degrOctaveBands, degrRate)
        coherences, cfreqs = octaveBandCoherence(refOctaveBands,
                                                 degrOctaveBands, refRate)
        thirdOctaveMTF = thirdOctaveRootSum(spectras, sfreqs)
        thirdOctaveCoherences = thirdOctaveRMS(coherences, cfreqs)

        # add to interim array for MTFs and coherences
        try:
            thirdOctaveTemps.append([thirdOctaveMTF, thirdOctaveCoherences])
        except:
            thirdOctaveTemps = [[thirdOctaveMTF, thirdOctaveCoherences]]

        print()
    
    # calculate the STI values
    print (" Speech Transmission Index ".center(80,'*'))
    for i in range(0,len(thirdOctaveTemps)):
        sampleSTI = sti(thirdOctaveTemps[i][0], thirdOctaveTemps[i][1])
        
        # add to STI output array
        try:
            stiValues.append(sampleSTI)
        except:
            stiValues = [sampleSTI]
    
    # unpack single value
    if len(stiValues) == 1:
        stiValues = stiValues[0]
    
    print()
    return stiValues

def readwav(path):
    """
    Reads Microsoft WAV format audio files, scales integer sample values and
    to [0,1]. Returns a tuple consisting of scaled WAV samples and sample rate
    in hertz.
    
    Input
    -----
    * path : string
    
        Valid system path to file
    
    Output
    ------
    * audio : array-like
    
        Array of scaled sampled
    
    * rate : int
    
        Audio sample rate in hertz
    """
    wav = wavfile.read(path)
    
    rate = wav[0]
    audio = array(wav[1])
    
    scale = float(max(audio))
    audio = audio / scale
    
    return audio, rate



from .formatConvert.wav_pcm import  wav2pcm
from .G160.G160 import cal_g160
from .P563.P563 import cal_563_mos
from .PESQ.PESQ import *
from .POLQA.polqa_client import polqa_client_test
from .SDR.SDR import cal_sdr
from .STI.cal_sti import cal_sti
from .STOI.STOI import *
from .resample.resampler import resample,restruct
from .timeAligment.time_align import cal_delay
from .commFunction import get_data_array,get_file_path
from .PCC.Pearson_CC import cal_PCC,get_max_cc_by_dll
from .SNR_ESTIMATION.MATCH_SIG import match_sig
from .SNR_ESTIMATION.SNR_MUSIC import cal_snr_music
from .SNR_ESTIMATION.SNR_TRANSIENT import cal_snr_transient
from .AGC_EVALUATION.CAL_GAIN_TABLE import cal_gain_table
from .AGC_EVALUATION.CAL_ATTACK_RELEASE import cal_attack_release
from .AGC_EVALUATION.CAL_MUSIC_STABILITY import cal_music_stablility
from .AGC_EVALUATION.CAL_DELAY import cal_DELAY
from .AEC_EVALUATION.MATCH_AEC import MATCH_AEC
from .AEC_EVALUATION.ERLE_ETSIMATION import cal_erle
from .FUNCTION.audioFunction import isSlience,audioFormat
from .AEC_MOS.aecmos import cal_aec_mos
from .MOS_INFER.run_predict import cal_mos_infer
from .Noise_Suppression.noiseFuction import  cal_noise_Supp
from operator import methodcaller
from .computeAudioQuality.mainProcess import computeAudioQuality

from ctypes import  *

def compute_audio_quality(metrics,testFile=None,refFile=None,micFile=None,cleFile=None,aecCaliFile=None,outFile=None,noiseFile=None,samplerate=16000,
                          bitwidth=2,channel=1,refOffset=0,testOffset=0,maxComNLevel =-48.0,speechPauseLevel=-35.0,audioType=0,
                          aecStartPoint=0,aecTargetType=0,aecScenario=0,rmsCalsection=None):
    """
    :param metrics: G160/P563/POLQA/PESQ/STOI/STI/PEAQ/SDR/SII/LOUDNESS/MUSIC/MATCH/
                    TRANSIENT/GAINTABLE/ATTACKRELEASE/MUSICSTA/AGCDELAY/MATCHAEC/
                    ERLE/SLIENCE/FORMAT/AECMOS/AIMOS/TRMS/ARMS/NOISE，必选项
    # G160 无采样率限制；  WAV/PCM输入 ；三端输入: clean、ref、test；无时间长度要求；
    # P563 8000hz(其他采样率会强制转换到8khz)；  WAV/PCM输入 ；单端输入: test；时长 < 20s；
    # POLQA 窄带模式  8k  超宽带模式 48k ；WAV/PCM输入 ；双端输入：ref、test；时长 < 20s；
    # PESQ 窄带模式  8k   宽带模式 16k ；WAV/PCM输入 ；双端输入：ref、test；时长 < 20s；
    # STOI 无采样率限制; 双端输入：ref、test；无时间长度要求；
    # STI >8k(实际会计算8khz的频谱)； WAV/PCM输入 ；双端输入：ref、test；时长 > 20s
    # PEAQ 无采样率限制；WAV/PCM输入 ；双端输入：ref、test；无时间长度要求；
    # SDR 无采样率限制; WAV/PCM输入 ; 双端输入：ref、test；无时间长度要求；
    # MATCH 无采样率限制; WAV/PCM输入;三端输入：ref、test、out； 无时间长度要求；
    # MUSIC 无采样率限制;WAV/PCM输入;双端输入：ref、test；无时间长度要求；
    # TRANSIENT 无采样率限制,WAV/PCM输入;三端输入：cle、noise、test； 无时间长度要求；
    # GAINTABLE 无采样率限制,WAV/PCM输入;双端输入：ref、test；固定信号输入；
    # ATTACKRELEASE 无采样率限制,WAV/PCM输入;双端输入：ref、test；固定信号输入；
    # MUSICSTA 无采样率限制,WAV/PCM输入;双端输入：ref、test；无时间长度要求；
    # AGCDELAY 无采样率限制,WAV/PCM输入;双端输入：ref、test；无时间长度要求；
    # MATCHAEC 无采样率限制 WAV/PCM输入;三端输入：ref、mic,test，；无时间长度要求；
    # ELRE 无采样率限制 WAV/PCM输入;三端输入：mic,ref、test；无时间长度要求；
    # SLIENCE 无采样率限制 WAV/PCM/MP4输入;单端输入：test；无时间长度要求；
    # FORMAT 无采样率限制 WAV/MP4输入;单端输入：test；无时间长度要求；
    # AECMOS 无采样率限制 WAV/PCM输入 ；三端输入：mic,ref、test；无时间长度要求；
    # AIMOS 无采样率限制 WAV/PCM输入 ；单端输入：test；无时间长度要求；
    # TRMS 无采样率限制 WAV/PCM输入 ；单端输入：test；无时间长度要求；
    # ARMS 无采样率限制 WAV/PCM输入 ；单端输入：test；无时间长度要求；
    # NOISE 无采样率限制 WAV/PCM输入 ；双端输入：ref、test；无时间长度要求；
    不同指标输入有不同的采样率要求，如果传入的文件不符合该指标的要求，会自动变采样到合法的区间
    :param testFile: 被测文件，必选项
    :param refFile:  参考文件，可选项，全参考指标必选，比如POLQA/PESQ/PEAQ
    :param micFile:  micIN，可选项，回声指标必选，MATCHAEC/ELRE/AECMOS
    :param cleFile:  干净语音文件，可选项，G160,TRANSIENT需要
    :param noiseFile 噪声文件，可选项，突发噪声信噪比计算需要
    :param aecCaliFile 用于做AEC对齐的校准文件  MATCHAEC专用
    :param outFile 输出文件，可选项，对齐文件可选
    :param samplerate: 采样率，可选项，pcm文件需要 default = 16000
    :param bitwidth: 比特位宽度，可选项，pcm文件需要 default = 2
    :param channel: 通道数，可选项，pcm文件需要 default = 1
    :param refOffset: ref文件的样点偏移，可选项，指标G160需要
    :param testOffset: test文件的样点偏移，可选项，指标G160需要
    :param maxComNLevel: 测试G160文件的最大舒适噪声
    :param speechPauseLevel 测试G160文件的语音间歇段的噪声
    :param audioType  输入音频的模式 0：语音 1：音乐 MATCH/GAINTABLE需要
    :param aecStartPoint  计算AECMOS，选择从第几秒开始计算
    :param aecTargetType  0:Chiness 1:English 2:Single Digit 3:Music  计算MATCHAEC/ELRE
    :param aecScenario 计算aec mos专用     0:'doubletalk_with_movement', 1:'doubletalk', 2:'farend_singletalk_with_movement', 3:'farend_singletalk', 4:'nearend_singletalk'
    :param rmsCalsection 计算rms的区间 TRMS和ARMS需要，时间单位s，比如：[1,20]
    :return:
    """
    paraDicts = {
        'metrics':metrics,
        'testFile':testFile,
        'refFile':refFile,
        'micFile':micFile,
        'cleFile':cleFile,
        'noiseFile':noiseFile,
        'aecCaliFile':aecCaliFile,
        'outFile':outFile,
        'samplerate':samplerate,
        'bitwidth':bitwidth,
        'channel':channel,
        'refOffset':refOffset,
        'testOffset':testOffset,
        'maxComNLevel':maxComNLevel,
        "speechPauseLevel":speechPauseLevel,
        "audioType":audioType,
        "aecStartPoint":aecStartPoint,
        "aecTargetType":aecTargetType,
        'aecScenario':aecScenario,
        'rmsCalsection':rmsCalsection
    }
    comAuQUA = computeAudioQuality(**paraDicts)
    return methodcaller(metrics)(comAuQUA)





from socket import *
import threading
import json
import os

address='10.219.33.45'
port=2159
buffsize=1024
s = socket(AF_INET, SOCK_STREAM)
s.bind((address,port))
s.listen(100)     #max connect
conn_list = []
conn_dt = {}
requeststack = {}

def tcplink(sock,addr):
    while True:
        try:
            recvdata=sock.recv(buffsize).decode('utf-8')
            curdic = json.loads(recvdata)
            if curdic['module'] == 'clientA' and curdic['method'] == 'requestA':
                requeststack[curdic['token']] = [0,'',[curdic['srcFile'],curdic['testFile'],curdic['samplerate']]]
                while True:
                    if requeststack[curdic['token']][1] != '':
                        oneresult = requeststack.pop(curdic['token'])[1]
                        curdic['result'] = oneresult
                        sock.send(bytes(json.dumps(curdic), encoding='utf-8'))
                        os.system("rm -rf /home/netease/polqa/" + curdic['token'])
                        break

            if curdic['module'] == 'clientB' and curdic['method'] == 'requestB':
                if len(requeststack) != 0:
                    for onekey in requeststack:
                        if requeststack[onekey][0] == 0:
                            requeststack[onekey][0] = 1
                            curdic['token'] =  onekey
                            curdic['job'] = 'occupy'
                            curdic['srcFile'] = requeststack[onekey][2][0]
                            curdic['testFile'] = requeststack[onekey][2][1]
                            curdic['samplerate'] = requeststack[onekey][2][2]
                            sock.send(bytes(json.dumps(curdic), encoding='utf-8'))
                            break
                    else:
                        curdic['job'] = None
                        sock.send(bytes(json.dumps(curdic),encoding='utf-8'))
                else:
                    curdic['job'] = None
                    sock.send(bytes(json.dumps(curdic),encoding='utf-8'))
            if curdic['module'] == 'clientB' and curdic['method'] == 'response':
                requeststack[curdic['token']][1] = curdic['result']
                sock.send(bytes(json.dumps(curdic),encoding='utf-8'))
            if not recvdata:
                break
        except:
            sock.close()
            print(addr,'offline')
            _index = conn_list.index(addr)
            conn_dt.pop(addr)
            conn_list.pop(_index)
            break

def recs():
    while True:
        clientsock,clientaddress=s.accept()
        if clientaddress not in conn_list:
            conn_list.append(clientaddress)
            conn_dt[clientaddress] = clientsock
        print('connect from:',clientaddress)
        #
        t=threading.Thread(target=tcplink,args=(clientsock,clientaddress))
        t.start()

if __name__ == '__main__':
    t1 = threading.Thread(target=recs, args=(), name='rec')

    t1.start()




# -*- coding: utf-8 -*-
"""
@author: Gabriel Mittag, TU-Berlin
"""

import os
import datetime


import pandas as pd; pd.options.mode.chained_assignment=None
import torch
import torch.nn as nn
from MOS_INFER import NISQA_lib as NL
import sys

sys.path.append('../')
class nisqaModel(object):
    '''
    nisqaModel: Main class that loads the model and the datasets. Contains
    the training loop, prediction, and evaluation function.                                               
    '''      
    def __init__(self, args):
        self.args = args
        self.runinfos = {}       
        self._getDevice()
        self._loadModel()
        self._loadDatasets()
        self.args['now'] = datetime.datetime.today()

    def predict(self):
        print('---> Predicting ...')
        if self.args['tr_parallel']:
            self.model = nn.DataParallel(self.model)           
        
        if self.args['dim']==True:
            y_val_hat, y_val = NL.predict_dim(
                self.model, 
                self.ds_val, 
                self.args['tr_bs_val'],
                self.dev,
                num_workers=self.args['tr_num_workers'])
        else:
            if self.args['task_type'] == 0:
                y_val_hat, y_val = NL.predict_mos(
                    self.model, 
                    self.ds_val, 
                    self.args['tr_bs_val'],
                    self.dev,
                    num_workers=self.args['tr_num_workers']) 
            elif self.args['task_type'] == 1:
                y_val_hat, y_val = NL.predict_mos_multitask(
                    self.model, 
                    self.ds_val, 
                    self.args['tr_bs_val'],
                    self.dev,
                    num_workers=self.args['tr_num_workers']) 
            elif self.args['task_type'] == 2:
                y_val_hat, y_val = NL.predict_mos_multifeature(
                    self.model, 
                    self.ds_val, 
                    self.args['tr_bs_val'],
                    self.dev,
                    num_workers=self.args['tr_num_workers']) 
            elif self.args['task_type'] == 3:
                y_val_hat, y_val = NL.predict_mos_multiresolution(
                    self.model_1, 
                    self.model_2, 
                    self.model_3, 
                    self.ds_val, 
                    self.args['tr_bs_val'],
                    self.dev,
                    num_workers=self.args['tr_num_workers']) 
            elif self.args['task_type'] == 4:
                y_val_hat, y_val = NL.predict_mos_multiscale(
                    self.model_1, 
                    self.model_2, 
                    self.model_3, 
                    self.ds_val, 
                    self.args['tr_bs_val'],
                    self.dev,
                    num_workers=self.args['tr_num_workers']) 
            
        # import pdb; pdb.set_trace()        
        if self.args['output_dir']:
            self.ds_val.df['model'] = self.args['name']
            self.ds_val.df.to_csv(
                os.path.join(self.args['output_dir'], 'test.csv'),
                index=False)

            
        # print(self.ds_val.df.to_string(index=False))

        if self.args['task_type'] == 1:
            r_mos = NL.calc_eval_metrics(y_val[:,0].squeeze(), y_val_hat[:,0].squeeze())
            r_std = NL.calc_eval_metrics(y_val[:,1].squeeze(), y_val_hat[:,1].squeeze())
            print('mos')
            print(r_mos)
            print('std')
            print(r_std)
        else:
            r = NL.calc_eval_metrics(y_val.squeeze(), y_val_hat.squeeze())
            print(r)

        return self.ds_val.df


    
    def _loadDatasets(self):
        if self.args['mode']=='predict_file':
            self._loadDatasetsFile()
        elif self.args['mode']=='predict_dir':
            self._loadDatasetsFolder()  
        elif self.args['mode']=='predict_csv':
            self._loadDatasetsCSVpredict()
        elif self.args['mode']=='main':
            self._loadDatasetsCSV()
        else:
            raise NotImplementedError('mode not available')                        
            
    

    def _loadDatasetsFile(self):
        data_dir = os.path.dirname(self.args['deg'])
        file_name = os.path.basename(self.args['deg'])
        df_val = pd.DataFrame([file_name], columns=['deg'])

        # creating Datasets ---------------------------------------------------
        self.ds_val = NL.SpeechQualityDataset(
            df_val,
            df_con=None,
            data_dir = data_dir,
            filename_column = 'deg',
            mos_column = 'predict_only',
            seg_length = self.args['ms_seg_length'],
            max_length = self.args['ms_max_segments'],
            to_memory = None,
            to_memory_workers = None,
            seg_hop_length = self.args['ms_seg_hop_length'],
            transform = None,
            ms_n_fft = self.args['ms_n_fft'],
            ms_hop_length = self.args['ms_hop_length'],
            ms_win_length = self.args['ms_win_length'],
            ms_n_mels = self.args['ms_n_mels'],
            ms_sr = self.args['ms_sr'],
            ms_fmax = self.args['ms_fmax'],
            ms_channel = self.args['ms_channel'],
            double_ended = self.args['double_ended'],
            dim = self.args['dim'],
            filename_column_ref = None,
            mos_std_column = 'mos_std',
            votes_column = 'votes',
            task_type = self.args['task_type'],
        )


    def _loadModel(self):    
        '''
        Loads the Pytorch models with given input arguments.
        '''   
        # if True overwrite input arguments from pretrained model
        # import pdb; pdb.set_trace()
        if self.args['pretrained_model']:
            # if os.path.isabs(self.args['pretrained_model']):
            #     model_path = os.path.join(self.args['pretrained_model'])
            # else:
            #     model_path = os.path.join(os.getcwd(), self.args['pretrained_model'])
            #model_path = os.path.join(sys.prefix, self.args['pretrained_model'])
            model_path = sys.prefix + '//'+ self.args['pretrained_model']
            print(sys.prefix,model_path)
            if self.args['task_type'] == 3 or self.args['task_type'] == 4:
                checkpoint = torch.load(model_path[:-4] + '_1.tar', map_location=self.dev)
                checkpoint_2 = torch.load(model_path[:-4] + '_2.tar', map_location=self.dev)
                checkpoint_3 = torch.load(model_path[:-4] + '_3.tar', map_location=self.dev)
                
            else:
                checkpoint = torch.load(model_path, map_location=self.dev)
            
            # update checkpoint arguments with new arguments
            checkpoint['args'].update(self.args)
            self.args = checkpoint['args']
            
        if self.args['model']=='NISQA_DIM':
            self.args['dim'] = True
            self.args['csv_mos'] = None # column names hardcoded for dim models
        else:
            self.args['dim'] = False
            
        if self.args['model']=='NISQA_DE':
            self.args['double_ended'] = True
        else:
            self.args['double_ended'] = False     
            self.args['csv_ref'] = None

        # Load Model
        self.model_args = {
            
            'ms_seg_length': self.args['ms_seg_length'],
            'ms_n_mels': self.args['ms_n_mels'],
            
            'cnn_model': self.args['cnn_model'],
            'cnn_c_out_1': self.args['cnn_c_out_1'],
            'cnn_c_out_2': self.args['cnn_c_out_2'],
            'cnn_c_out_3': self.args['cnn_c_out_3'],
            'cnn_kernel_size': self.args['cnn_kernel_size'],
            'cnn_dropout': self.args['cnn_dropout'],
            'cnn_pool_1': self.args['cnn_pool_1'],
            'cnn_pool_2': self.args['cnn_pool_2'],
            'cnn_pool_3': self.args['cnn_pool_3'],
            'cnn_fc_out_h': self.args['cnn_fc_out_h'],
            
            'td': self.args['td'],
            'td_sa_d_model': self.args['td_sa_d_model'],
            'td_sa_nhead': self.args['td_sa_nhead'],
            'td_sa_pos_enc': self.args['td_sa_pos_enc'],
            'td_sa_num_layers': self.args['td_sa_num_layers'],
            'td_sa_h': self.args['td_sa_h'],
            'td_sa_dropout': self.args['td_sa_dropout'],
            'td_lstm_h': self.args['td_lstm_h'],
            'td_lstm_num_layers': self.args['td_lstm_num_layers'],
            'td_lstm_dropout': self.args['td_lstm_dropout'],
            'td_lstm_bidirectional': self.args['td_lstm_bidirectional'],
            
            'td_2': self.args['td_2'],
            'td_2_sa_d_model': self.args['td_2_sa_d_model'],
            'td_2_sa_nhead': self.args['td_2_sa_nhead'],
            'td_2_sa_pos_enc': self.args['td_2_sa_pos_enc'],
            'td_2_sa_num_layers': self.args['td_2_sa_num_layers'],
            'td_2_sa_h': self.args['td_2_sa_h'],
            'td_2_sa_dropout': self.args['td_2_sa_dropout'],
            'td_2_lstm_h': self.args['td_2_lstm_h'],
            'td_2_lstm_num_layers': self.args['td_2_lstm_num_layers'],
            'td_2_lstm_dropout': self.args['td_2_lstm_dropout'],
            'td_2_lstm_bidirectional': self.args['td_2_lstm_bidirectional'],                
            
            'pool': self.args['pool'],
            'pool_att_h': self.args['pool_att_h'],
            'pool_att_dropout': self.args['pool_att_dropout'],
            }
            
        if self.args['double_ended']:
            self.model_args.update({
                'de_align': self.args['de_align'],
                'de_align_apply': self.args['de_align_apply'],
                'de_fuse_dim': self.args['de_fuse_dim'],
                'de_fuse': self.args['de_fuse'],        
                })
                      
        print('Model architecture: ' + self.args['model'])
        if self.args['model']=='NISQA':
            if self.args['task_type'] == 0:
                self.model = NL.NISQA(**self.model_args)     
            elif self.args['task_type'] == 1:
                self.model = NL.NISQA_MULTITASK(**self.model_args)
            elif self.args['task_type'] == 2:
                self.model = NL.NISQA(**self.model_args) 
            elif self.args['task_type'] == 3:
                self.model_1 = NL.NISQA(**self.model_args)  
                self.model_2 = NL.NISQA(**self.model_args) 
                self.model_3 = NL.NISQA(**self.model_args)
            elif self.args['task_type'] == 4:
                self.model_1 = NL.NISQA(**self.model_args)  
                self.model_2 = NL.NISQA(**self.model_args) 
                self.model_3 = NL.NISQA(**self.model_args) 
                
        elif self.args['model']=='NISQA_DIM':
            self.model = NL.NISQA_DIM(**self.model_args)     
        elif self.args['model']=='NISQA_DE':
            self.model = NL.NISQA_DE(**self.model_args)     
        else:
            raise NotImplementedError('Model not available')                        
        
        
        # Load weights if pretrained model is used ------------------------------------
        if self.args['pretrained_model']:
            if self.args['task_type'] == 3 or self.args['task_type'] == 4:
                missing_keys, unexpected_keys = self.model_1.load_state_dict(checkpoint['model_state_dict'], strict=True)
                missing_keys, unexpected_keys = self.model_2.load_state_dict(checkpoint_2['model_state_dict'], strict=True)
                missing_keys, unexpected_keys = self.model_3.load_state_dict(checkpoint_3['model_state_dict'], strict=True)
            else:
                missing_keys, unexpected_keys = self.model.load_state_dict(checkpoint['model_state_dict'], strict=True)
            print('Loaded pretrained model from ' + self.args['pretrained_model'])
            if missing_keys:
                print('missing_keys:')
                print(missing_keys)
            if unexpected_keys:
                print('unexpected_keys:')
                print(unexpected_keys)  
                
        # para_num = sum([p.numel() for p in self.model.parameters()])
        # para_size = para_num * 4 / 1024
        # import pdb; pdb.set_trace()


    def _getDevice(self):
        '''
        Train on GPU if available.
        '''
        if torch.cuda.is_available():
            self.dev = torch.device("cuda")
        else:
            self.dev = torch.device("cpu")

        if "tr_device" in self.args:
            if self.args['tr_device']=='cpu':
                self.dev = torch.device("cpu")
            elif self.args['tr_device']=='cuda':
                self.dev = torch.device("cuda")
        print('Device: {}'.format(self.dev))

        if "tr_parallel" in self.args:
            if (self.dev==torch.device("cpu")) and self.args['tr_parallel']==True:
                self.args['tr_parallel']==False
                print('Using CPU -> tr_parallel set to False')


            



from pystoi.stoi import stoi
import numpy as np

def cal_stoi(ref, est, sr=16000):
    return stoi(ref, est, sr, extended=False)


if __name__ == '__main__':
    with open('clean.pcm','rb') as ref:
        pcmdata = ref.read()
    with open('in.pcm','rb') as ref:
        indata = ref.read()
    ref = np.frombuffer(pcmdata,dtype=np.int16)
    ins = np.frombuffer(indata, dtype=np.int16)
    print(cal_stoi(ref,ins))


# -*- coding: UTF-8 -*-
import sys
sys.path.append('../')
from ctypes import *
import ctypes
import numpy as np
#  * -------------------------------------------------------------------------
#  * Arguments    : const emxArray_real_T *ref_in
#  *                const emxArray_real_T *sig_in
#  *                double fs
#  *                double *delay
#  *                double *err
#  * Return Type  : void
#  */
# void delay_estimation_15s(const emxArray_real_T *ref_in, const emxArray_real_T
#   *sig_in, double fs, double *delay, double *err)

def cal_PCC(refData, testData,calSize,mydll):
    """
    """

    refstruct = np.ascontiguousarray(refData, dtype=c_double).ctypes.data_as(ctypes.POINTER(c_double))
    teststruct = np.ascontiguousarray(testData, dtype=c_double).ctypes.data_as(ctypes.POINTER(c_double))
    mydll.Pearson_Correlation.argtypes = [POINTER(c_double),POINTER(c_double),c_long,POINTER(c_double)]
    pcc= c_double(0.0)
    mydll.Pearson_Correlation(refstruct,teststruct,c_long(calSize),byref(pcc))
    return pcc.value

def get_max_cc_by_dll(refData, testData,mydll,step):
    """
    (double *x, double *y,long xLenth,long yLenth,long step,double *maxcc,long *startPoint)
    """

    refstruct = np.ascontiguousarray(refData, dtype=c_double).ctypes.data_as(ctypes.POINTER(c_double))
    teststruct = np.ascontiguousarray(testData, dtype=c_double).ctypes.data_as(ctypes.POINTER(c_double))
    mydll.get_max_pcc.argtypes = [POINTER(c_double),POINTER(c_double),c_long,c_long,c_long,POINTER(c_double),POINTER(c_long)]
    max_cc,start_point = c_double(0.0),c_long(0)
    mydll.get_max_pcc(refstruct,teststruct,c_long(len(refData)),c_long(len(testData)),c_long(step),byref(max_cc),byref(start_point))
    return max_cc.value,start_point.value


if __name__ == '__main__':
    ref = r'C:\Users\vcloud_avl\Documents\我的POPO\src.wav'
    test = r'C:\Users\vcloud_avl\Documents\我的POPO\test.wav'
    from commFunction import  get_data_array_double
    import platform,time
    newdata,fs,ch = get_data_array_double(ref)
    refdata,fs,ch = get_data_array_double(test)
    newdata = newdata[:10000]
    refdata = refdata[:10000]
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/pcc.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/pcc.dylib')
    #mydll.Pearson_Correlation.argtypes = [POINTER(c_double), POINTER(c_double), c_long, POINTER(c_double)]
    # newdata = np.arange(0,20000,2)
    # refdata = np.arange(10000,30000,2)
    # print(len(newdata),len(refdata))
    # print(time.time())
    # print(np.corrcoef(refdata,newdata))
    # print(time.time())
    # print(cal_PCC(refdata,newdata,10000,mydll))
    # print(time.time())
    suma,sumb = 0,0
    for e in range(10000):
        a = time.time()
        res = np.corrcoef(refdata, newdata)
        print(res)
        b = time.time()
        suma += b - a
        c = time.time()
        res = cal_PCC(refdata, newdata, 10000, mydll)
        print(res)
        d = time.time()
        sumb += d - c
    print(suma/10000)
    print(sumb/10000)
    pass


# void music_stability(const emxArray_real_T *ref, const emxArray_real_T *sig,
#                      double fs_ref, double fs_sig, double music_stability_ratio
#                      [1294], double *err)

# -*- coding: UTF-8 -*-
import sys

sys.path.append('../')
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_
import ctypes

def cal_music_stablility(refFile=None, testFile=None):
    """
    """
    refstruct,refsamplerate,_ = get_data_of_ctypes_(refFile,True)
    teststruct,testsamplerate,_ = get_data_of_ctypes_(testFile,True)

    if refsamplerate != testsamplerate :
        raise TypeError('Different format of ref and test files!')

    import platform
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/musicStability.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/musicStability.dylib')

    mydll.music_stability.argtypes = [POINTER(emxArray_real_T),POINTER(emxArray_real_T),c_double,c_double,POINTER(c_double),POINTER(c_double)]
    data_format = c_double*1294
    msr = data_format()
    err = c_double(0.0)
    mydll.music_stability(byref(refstruct),byref(teststruct),c_double(refsamplerate),c_double(refsamplerate),msr,byref(err))

    if err.value == 0.0:
        return msr
    else:
        return None


if __name__ == '__main__':
    file = r'C:\Users\vcloud_avl\Documents\我的POPO\0\music_stability.wav'
    test = r'C:\Users\vcloud_avl\Documents\我的POPO\0\music_stability.wav'
    res = cal_music_stablility(refFile=file,testFile=test)
    for a in res:
        print(a)
    pass


import numpy as np


def cal_sdr(reference, estimation, sr=16000):
    """
    Scale-Invariant Signal-to-Distortion Ratio (SI-SDR)

    Args:
        reference: numpy.ndarray, [..., T]
        estimation: numpy.ndarray, [..., T]

    Returns:
        SI-SDR

    References
        SDR– Half- Baked or Well Done? (http://www.merl.com/publications/docs/TR2019-013.pdf)
    """
    estimation, reference = np.broadcast_arrays(estimation, reference)
    reference_energy = np.sum(reference ** 2, axis=-1, keepdims=True)

    optimal_scaling = np.sum(reference * estimation, axis=-1, keepdims=True) / reference_energy

    projection = optimal_scaling * reference

    noise = estimation - projection

    ratio = np.sum(projection ** 2, axis=-1) / np.sum(noise ** 2, axis=-1)
    return 10 * np.log10(ratio)


if __name__ == '__main__':
    with open('clean.pcm','rb') as ref:
        pcmdata = ref.read()
    with open('in.pcm','rb') as ref:
        indata = ref.read()
    ref = np.frombuffer(pcmdata,dtype=np.int16)
    ins = np.frombuffer(indata, dtype=np.int16)
    print(cal_sdr(ref,ins))


import ctypes
import sys
from ctypes import *
from formatConvert import wav2pcm


class delaystruct(Structure):
 _fields_ = [

        ("Crude_DelayEst", c_long),  # c_byte

 ]

def cal_delay(reffile, testfile, samplerate):
        delays = delaystruct()

        import platform
        mydll = None
        cur_paltform = platform.platform().split('-')[0]
        if cur_paltform == 'Windows':
            mydll = ctypes.windll.LoadLibrary(sys.prefix + '/time_align.dll')
        if cur_paltform == 'macOS':
            mydll = CDLL(sys.prefix + '/time_align.dylib')

        inputFile = c_char_p(
               bytes(reffile.encode('utf-8')))  # create_unicode_buffer(inFile.encode('utf-8'), len(inFile))
        outputFile = c_char_p(
               bytes(testfile.encode('utf-8')))  # create_unicode_buffer(outFile.encode('utf-8'), len(outFile))
        if samplerate == 8000:
               mode = 'nb'
        if samplerate == 16000:
               mode = 'wb'
        cmode = c_char_p(bytes(mode.encode('utf-8')))
        mydll.cal_delay(inputFile, outputFile, samplerate, cmode, byref(delays))
        print(delays.Crude_DelayEst)
        return delays.Crude_DelayEst

if __name__ == '__main__':
       ref = r'C:\Users\vcloud_avl\Documents\我的POPO\src.wav'
       test = r'C:\Users\vcloud_avl\Documents\我的POPO\test.wav'
       cal_delay(wav2pcm(ref),wav2pcm(test),48000)
       pass